
// **********************************************************************
// *                                                                    *
// * Common code file for the SMBUS on the ALI M1535 chipset.           *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************


/*----------------------* PRAGMAS *----------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*----------------------* DEFINES *----------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*---------------------*  HEADERS *----------------------------*/
/*-------------------------------------------------------------*/
#include "i2cos.h"
#include "i2cio.h"
#include "i2cali.h"
#include "pci.h"

/*--------------------* LOCAL DATA *---------------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*--------------------* EXTERNAL DATA *------------------------*/
/*-------------------------------------------------------------*/
extern int MAX_RETRIES;
extern unsigned int SMBusBaseAddr;
extern int IsOSB4Found;

/*----------------* EXTERNAL PROTOTYPES *----------------------*/
/*-------------------------------------------------------------*/

// None ---------

/*-------------------* LOCAL PROTOTYPES *----------------------*/
/*-------------------------------------------------------------*/
I2CSTATUS ALIWaitForSMBus(void);
I2CSTATUS ALITestSMBusStatus(void);
I2CSTATUS ALIReceiveByte(BYTE, BYTE FAR *);
I2CSTATUS ALISendByte(BYTE, BYTE);
I2CSTATUS ALIReadDataByte(BYTE, BYTE, BYTE FAR *);
I2CSTATUS ALIWriteDataByte(BYTE, BYTE, BYTE);
I2CSTATUS ALIReadDataWord(BYTE, BYTE, short FAR *);
I2CSTATUS ALIWriteDataWord(BYTE, BYTE, short);
I2CSTATUS ALIReadDataBlock(BYTE, BYTE, BYTE *, int *);
I2CSTATUS ALIWriteDataBlock(BYTE, BYTE, BYTE *, int);
I2CSTATUS ALIQuickCommand(BYTE, BYTE);

/*-----------------------* FUNCTIONS *-------------------------*/
/*-------------------------------------------------------------*/
///////////////////////////////////////////////////////////////////////////
// ALI Wait for SMBus                                       	         //
//									                                     //
// Notes: Waits for ALI SMBus host and slave status register busy bits   //
//        to become 0 (free).                                            //
//									                                     //
// On Entry:   None.                                            	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIWaitForSMBus(void)
{
 int iCount = 0;

 while ((INP (SMBusBaseAddr + ALI_SMB_STA) & ALI_SMB_STA_HST_BUSY))
       {
        if (iCount == MAX_DELAY)
            return (ERR_TIMEOUT);

        DELAY (DELAY_TIME);
        iCount++;
       }

 // Check for the SMBus idle condition
 if (!(INP (SMBusBaseAddr + ALI_SMB_STA) & ALI_SMB_STA_BUS_IDLE))
     return (ERR_BUS_ERROR);

 // Clear all status bits
 // Note:  This is generally unnecessary as the SMBus is ready for use at this point
 //        however, a 1 bit written to any bit position but bit 0 will clear that bit
 //        position.
 OUTP (SMBusBaseAddr + ALI_SMB_STA, INP (SMBusBaseAddr + ALI_SMB_STA));

 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// ALI Test SMBus Status                                       	         //
//									                                     //
// Notes: Checks SMBus host status register for completion of the        //
//        requested transaction.                                         //
//									                                     //
// On Entry:   None.                                            	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALITestSMBusStatus(void)
{
 int iCount = 0;
 BYTE Status;

 // Wait for host to complete the command
 while (!(Status = INP (SMBusBaseAddr + ALI_SMB_STA) & ALI_SMB_STA_CMD_DONE))
       {
        if (iCount == MAX_DELAY)
            {
             // Kill the current transaction
             OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_KILL);

             // Clear all status bits
             OUTP (SMBusBaseAddr + ALI_SMB_STA, INP (SMBusBaseAddr + ALI_SMB_STA));

             return (ERR_TIMEOUT);
            }

        DELAY (DELAY_TIME);
        iCount++;
       }

 // Wait for the SMBus idle condition
 while (!(Status = INP (SMBusBaseAddr + ALI_SMB_STA) & ALI_SMB_STA_BUS_IDLE))
       {
        if (iCount == MAX_DELAY)
           {
            // Kill the current transaction
            OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_KILL);

            // Clear all status bits
            OUTP (SMBusBaseAddr + ALI_SMB_STA, INP (SMBusBaseAddr + ALI_SMB_STA));

            return (ERR_TIMEOUT);
           }

        DELAY (DELAY_TIME);
        iCount++;
       }

 // Clear all status bits
 OUTP (SMBusBaseAddr + ALI_SMB_STA, INP (SMBusBaseAddr + ALI_SMB_STA));

 // Check for successful completion, if not return proper error code
 if ((Status & 0xE0) != 0x00)
    {
     if (Status & 0x80)
         return (ERR_FAILED);

     if (Status & 0x40)
         return (ERR_BUS_ERROR);

     if (Status & 0x20)
         return (ERR_DEV_ERROR);
    }

 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Read                                            	         //
//									                                     //
// Notes: Common read routine for ALI on the SMBus. Includes dispatch to //
//        Receive Byte and Read byte/word protocol handlers.             //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusRead(BYTE Address, BYTE Command, void FAR *BufferPtr, int Length, int RetryCount)
{
 BYTE ReturnByte;
 short ReturnWord;
 I2CSTATUS Status;
 int iCountIndex;

 // Initialize the user output buffer
 for (iCountIndex = 0; iCountIndex < Length; iCountIndex++)
      ((char FAR*)BufferPtr)[iCountIndex] = 0;

 // Check length indicating a read word request 
 if (Length == 2)
    {
     // Do the read word protocol
     Status = ALIReadDataWord (Address, Command, &ReturnWord);

     // If an error occured setup for a retry
     if (Status != ERR_NO_ERROR)
        {
         if (RetryCount >= MAX_RETRIES)
             return (Status);
         else
            {
             DELAY (10000);

             // Retry by calling ourself
             return ALI_SMBusRead (Address, Command, BufferPtr, Length, RetryCount + 1);
            }
        }
     // Save the returned word in the user buffer
     ((short FAR*)BufferPtr)[0] = ReturnWord;
    }
 else                     
    {
     // Do the read byte protocol
     Status = ALIReadDataByte(Address, Command, &ReturnByte);

     // If an error occured setup for a retry
     if (Status != ERR_NO_ERROR)
        {
         if (RetryCount >= MAX_RETRIES)
             return (Status);
         else
           {
            DELAY (10000);

            // Retry by calling ourself
            return ALI_SMBusRead (Address, Command, BufferPtr, Length, RetryCount + 1);
           }
        }

     // Save the returned byte in the user buffer
     ((char FAR *)BufferPtr)[0] = ReturnByte;

     // Implement multi-byte read as a 'send command then read byte' followed
     // by multiple 'receive byte' operations.  This should work fine for 
     // EEPROMs and devices which implement 'auto increment' registers. 
     for (iCountIndex = 1; iCountIndex < Length; iCountIndex++)
         {
         // Do the receive byte protocol
          Status = ALIReceiveByte(Address, &ReturnByte);

          // If an error occured setup for a retry
          if (Status != ERR_NO_ERROR)
             {
              if (RetryCount >= MAX_RETRIES)
                  return (Status);
              else
                 {
                  DELAY (10000);
               
                  // Retry by calling ourself
                  return ALI_SMBusRead(Address, Command, BufferPtr, Length, RetryCount + 1);
                 }
             }

          // Save the returned byte in the user buffer
          ((char FAR *)BufferPtr)[iCountIndex] = ReturnByte;
         }
    }

 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Write                                            	         //
//									                                     //
// Notes: Common write routine for ALI on the SMBus. Includes dispatch   //
//        to Write byte/word protocol handlers.                          //
//									                                     //
// On Entry:   Address, command, pointer to write buffer, buffer length  //
//             and retry count.                                          //
 //								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusWrite(BYTE Address, BYTE Command, void FAR *BufferPtr, int Length, int RetrtCount)
{
 I2CSTATUS Status;
 int iIndex, iCount;
   
 // Check length indicating a write word request 
 if (Length == 2)
    {
     // Do the write word protocol
     Status = ALIWriteDataWord (Address, Command, ((short FAR *)BufferPtr)[0]);

     // If an error occured setup for a retry
     if (Status != ERR_NO_ERROR)
        {
         if (RetrtCount >= MAX_RETRIES)
             return (Status);
         else
            {
             DELAY (10000);

             // Retry by calling ourself
             return ALI_SMBusWrite (Address, Command, BufferPtr, Length, RetrtCount + 1);
            }
        }
    }
 else
    {
     for (iIndex = 0; iIndex < Length; iIndex++)
         {
          // Do the write byte protocol
          Status = ALIWriteDataByte (Address, Command, ((char FAR *)BufferPtr)[iIndex]);
   
          // If an error occured setup for a retry
          iCount = 0;
          while (Status != ERR_NO_ERROR)
                {
                 iCount++;

                 if (iCount == MAX_RETRIES)
                    return (Status);
                 else
                    {
                     DELAY (10000);

                     // Do the write byte protocol
                     Status = ALIWriteDataByte (Address, Command, ((char FAR *)BufferPtr)[iIndex]);
                    }
                }
         
          // Increment the 'command' field.  This makes multi-byte writes 
          // with length > 2 work on EEPROMs and LMxx.  
          Command++;
         }
    }

 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Receive                                        	         //
//									                                     //
// Notes: Control routine for the Receive Byte protocol handler.         //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusReceive(BYTE Address, void FAR *BufferPtr, int Length, int RetryCount)
{
 BYTE ReturnByte;
 I2CSTATUS Status;
 int iIndex, iCount;

 // Initialize the user output buffer
 for (iIndex = 0; iIndex < Length; iIndex++)
      ((char FAR*)BufferPtr)[iIndex] = 0;

 for (iIndex = 0; iIndex < Length; iIndex++)
     {
     // Do the receive byte protocol
      Status = ALIReceiveByte (Address, &ReturnByte);

      // If an error occured setup for a retry
      iCount = 0;
      while (Status != ERR_NO_ERROR)
            {
             ++iCount;

             if (iCount == MAX_RETRIES)
                 return (Status);
             else
               {
                DELAY (10000);

                // Do the receive byte protocol
                Status = ALIReceiveByte (Address, &ReturnByte);
               }
            }

      // Save the returned byte in the user buffer
      ((char FAR *)BufferPtr)[iIndex] = ReturnByte;
     }

 return (ERR_NO_ERROR);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Send                                           	         //
//									                                     //
// Notes: Control routine for the Send Byte protocol handler.            //
//									                                     //
// On Entry:   Address and command.                                      //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusSend(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 Status = ALISendByte(Address, Command);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Read Block                                      	         //
//									                                     //
// Notes: Control routine for the Block Read protocol handler.           //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusReadBlock(BYTE Address, BYTE Command, BYTE FAR *BufferPtr, int *Length, int RetryCount)
{
 I2CSTATUS Status;

 Status = ALIReadDataBlock (Address, Command, BufferPtr, Length);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Write Block                                     	         //
//									                                     //
// Notes: Control routine for the Block Write protocol handler.          //
//									                                     //
// On Entry:   Address, command, pointer to read buffer, buffer length   //
//             and retry count.                                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusWriteBlock(BYTE Address, BYTE Command, BYTE FAR *BufferPtr, int Length, int RetryCount)
{
 I2CSTATUS Status;

 Status = ALIWriteDataBlock (Address, Command, BufferPtr, Length);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI SMBus Quick Command                                  	         //
//									                                     //
// Notes: Control routine for the Quick Command protocol handler.        //
//									                                     //
// On Entry:   Address, command.                                         //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALI_SMBusQuickCommand(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 Status = ALIQuickCommand (Address, Command);

 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI Receive Byte                                          	         //
//									                                     //
// Notes: Reads data from the SMBus address in receive byte mode.        //
//									                                     //
// On Entry:   Address, pointer to read buffer.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIReceiveByte(BYTE Address, BYTE FAR* ReadByte)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address | 0x01));

 // Setup the host control register (byte receive mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_SND_RCV);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = ALITestSMBusStatus ()) != ERR_NO_ERROR)
     return (Status);

 // Get the read byte
 *ReadByte = INP (SMBusBaseAddr + ALI_SMB_DEV_D0);

 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI Send Byte                                             	         //
//									                                     //
// Notes: Writes data to the SMBus address in send byte mode.            //
//									                                     //
// On Entry:   Address and command.                               	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALISendByte(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with write transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address & 0xFE));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the host control register (byte send mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_SND_RCV);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Return the transaction status
 return (ALITestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// ALI Read Data Byte                                       	         //
//									                                     //
// Notes: Reads data from the SMBus address in byte mode.                //
//									                                     //
// On Entry:   Address, command, pointer to read buffer.                 //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIReadDataByte(BYTE Address, BYTE Command, BYTE FAR *ReadByte)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address | 0x01));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the host control register (byte mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_BYTE);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = ALITestSMBusStatus ()) != ERR_NO_ERROR)
     return (Status);

 // Get the read byte
 *ReadByte = INP (SMBusBaseAddr + ALI_SMB_DEV_D0);

 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI Write Data Byte                                       	         //
//									                                     //
// Notes: Writes data to the SMBus address in byte mode.                 //
//									                                     //
// On Entry:   Address, command, byte to write.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIWriteDataByte(BYTE Address, BYTE Command, BYTE WriteByte)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with write transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address & 0xFE));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the host control register (byte mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_BYTE);

 // Setup the data register 0 with the write byte
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_D0, WriteByte);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Return the transaction status
 return (ALITestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// ALI Read Data Word                                       	         //
//									                                     //
// Notes: Reads data from the SMBus address in word mode.                //
//									                                     //
// On Entry:   Address, command, pointer to read buffer.                 //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIReadDataWord(BYTE Address, BYTE Command, short FAR *ReadWord)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address | 0x01));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the host control register (word mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_WORD);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = ALITestSMBusStatus ()) != ERR_NO_ERROR)
     return (Status);

 // Get the lower 8 bits of the read word
 ((char FAR*)ReadWord)[0] = INP (SMBusBaseAddr + ALI_SMB_DEV_D0);

 // Get the upper 8 bits of the read word
 ((char FAR*)ReadWord)[1] = INP (SMBusBaseAddr + ALI_SMB_DEV_D1);

 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI Write Data Word                                       	         //
//									                                     //
// Notes: Writes data to the SMBus address in word mode.                 //
//									                                     //
// On Entry:   Address, command, word to write.                          //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIWriteDataWord(BYTE Address, BYTE Command, short WriteWord)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with write transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address & 0xFE));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the host control register (word mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_WORD);

 // Setup the data register 0 with the lower 8 bits of the write word
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_D0, ((char FAR *)&WriteWord)[0]);

 // Setup the data register 1 with the upper 8 bits of the write word
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_D1, ((char FAR *)&WriteWord)[1]);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Return the transaction status
 return (ALITestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// ALI Read Data Block                                      	         //
//									                                     //
// Notes: Reads data from the SMBus address in block mode.               //
//									                                     //
// On Entry:   Address, command, pointer to read data buffer and the     //
//             buffer length.                                      	     //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIReadDataBlock(BYTE Address, BYTE Command, BYTE *Data, int *Length)
{
 I2CSTATUS Status;
 int iIndex = 0;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with read transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address | 0x01));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the host control register (block mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_BLOCK);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Check to the see the transaction completed successfully, if not return the error
 if ((Status = ALITestSMBusStatus ()) != ERR_NO_ERROR)
     return (Status);

 // Get the length of the data to be read
 *Length = INP (SMBusBaseAddr + ALI_SMB_DEV_D0);

 // Reset the block data register (data pointer) by writing to the host status register
 OUTP (SMBusBaseAddr + ALI_SMB_STA, ALI_SMB_STA_CLR_INDX);

 // Get the data from the block data register
 while (iIndex < *Length)
       {
		Data[iIndex] = INP (SMBusBaseAddr + ALI_SMB_DEV_BLK_DAT);
		iIndex++;
	   }

 // Return the transaction status
 return (Status);
}

///////////////////////////////////////////////////////////////////////////
// ALI Write Data Block                                      	         //
//									                                     //
// Notes: Writes data to the SMBus address in block mode.                //
//									                                     //
// On Entry:   Address, command, pointer to write data buffer and buffer //
//             length.                                             	     //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIWriteDataBlock(BYTE Address, BYTE Command, BYTE *Data, int Length)
{
 I2CSTATUS Status;
 int iIndex = 0;

 // Check the buffer length and adjust if necessary to the proper length.
 // NOTE:  In proper practice this test should just return an error code to the application.
 if (Length > 32)
     Length = 32;

 if (Length < 0)
     Length = 0;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address with write transfer
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, (Address & 0xFE));

 // Setup the slave command register
 OUTP (SMBusBaseAddr + ALI_SMB_SLV_CMD, Command);

 // Setup the number of bytes to write
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_D0, Length); 

 // Reset the block data register (data pointer) by writing to the host status register
 OUTP (SMBusBaseAddr + ALI_SMB_STA, ALI_SMB_STA_CLR_INDX);

 // Load the data into the block data register
 while (iIndex < Length)
       {
        OUTP (SMBusBaseAddr + ALI_SMB_DEV_BLK_DAT, Data[iIndex]);
        iIndex++;
       }

 // Setup the host control register (block mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_BLOCK);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Return the transaction status
 return (ALITestSMBusStatus ());
}

///////////////////////////////////////////////////////////////////////////
// ALI Quick Command                                           	         //
//									                                     //
// Notes: Writes data to the SMBus address in quick command mode.        //
//									                                     //
// On Entry:   Address, command.                                         //
//								                                         //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS ALIQuickCommand(BYTE Address, BYTE Command)
{
 I2CSTATUS Status;

 // Check to see if we can use the SMBus, if not return the error
 if ((Status = ALIWaitForSMBus ()) != ERR_NO_ERROR)
     return (Status);

 // Setup the slave device address (Note: address has the command embedded in it)
 OUTP (SMBusBaseAddr + ALI_SMB_DEV_ADD, Address);

 // Setup the host control register (quick mode)
 OUTP (SMBusBaseAddr + ALI_SMB_CMD_TYP, ALI_SMB_TYP_QUICK);

 // Issue the start command to the host controler
 OUTP (SMBusBaseAddr + ALI_SMB_STR_PORT, 1);

 // Return the transaction status
 return (ALITestSMBusStatus ());
}
