
// **********************************************************************
// *                                                                    *
// * Code to perform communication with the I2C on the ICH Chip.        *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

//  the CLEAR_HOST_STATUS must be changed from PIIX4 version because of 
//  INUSE_STS bit in SMB status reg.  This affects all of the PIIX4 functions 
//  so same functions must be defined for ICH.  


#include "i2cos.h"
#include "i2cio.h"
#include "pci.h"
#include "i2cich.h"

extern int MAX_RETRIES;

#define LAST_BYTE             0x20
#define DELAY_FOR_IO          500
#define SHORT_DELAY_FOR_IO    15
#define ICH_WAIT_DELAY_LINUX  50

// ------------------------------
//  Internal Function Prototypes
// ------------------------------
I2CSTATUS ICHWaitForSMBus(void);
I2CSTATUS ICHTestSMBusStatus(void);
I2CSTATUS ICHReceiveByte(BYTE IN addr, BYTE FAR* OUT retval);
I2CSTATUS ICHReadDataByte(BYTE slave_addr, BYTE command_code, BYTE FAR *byte_read);
I2CSTATUS ICHWriteDataByte(BYTE slave_addr, BYTE command_code, BYTE byte_written);
I2CSTATUS ICHReadDataWord(BYTE IN addr, BYTE IN pointer, short FAR OUT *retval);
I2CSTATUS ICHWriteDataWord(BYTE IN addr, BYTE IN pointer, short IN value);
I2CSTATUS ICHReadDataBlock1(BYTE IN address, BYTE IN command, BYTE OUT *data, int *length);
I2CSTATUS ICHWriteDataBlock1(BYTE IN address, BYTE IN command, BYTE IN *data, int length);
I2CSTATUS ICHSendCommand(BYTE IN address, BYTE IN command);

//-------------------
// External global variables
//-------------------
extern unsigned int SMBusBaseAddr;


// ---------------------------------------------------------------------------

// Wait INUSE_STS is free.  If INUSE=1, bus is in use.  First read
// when INUSE=0 forces INUSE=1;
// Returns ERR_NO_ERROR on success.
I2CSTATUS ICHWaitForSMBus()
{  
   int j;

// Wait for the INUSE semaphore to be 0, and acquire semaphore
   j = 0;
   debug_out(DETAILS_LVL,"ICHWaitForSMBus.  Wait for ICH_SMB_INUSE semaphore.\n");
   while (INP(SMBusBaseAddr + ICH_HST_STA) & ICH_SMB_INUSE)
   {
      if (j == ICH_WAIT_DELAY_LINUX)
	  {
	     debug_out(DETAILS_LVL,"ICHWaitForSMBus. Clear ICH_SMB_INUSE semaphore. Return ERR_TIMEOUT.\n");
         // Be sure to clear semaphore before return
         ICH_CLEAR_HOST_STATUS();
         return ERR_TIMEOUT;
	  }
      else
      {
         DELAY(DELAY_TIME);
         j++;
      }
   }

   debug_out(DETAILS_LVL,"ICHWaitForSMBus.  Wait for ICH_SMB_HOSTBUSY.\n");
// Wait for the busy bit
   j = 0;
   while (INP(SMBusBaseAddr + ICH_HST_STA) & ICH_SMB_HOSTBUSY)
   {
      if (j == MAX_DELAY)
	  {
         debug_out(DETAILS_LVL,"ICHWaitForSMBus.  Clear ICH_SMB_HOSTBUSY semaphore.  Return ERR_TIMEOUT\n");
        // Be sure to clear semaphore before return
         ICH_CLEAR_HOST_STATUS();
         return ERR_TIMEOUT;
	  }
      else
      {
         DELAY(DELAY_TIME);
         j++;
      }
   }

   debug_out(DETAILS_LVL,"ICHWaitForSMBus.  Clear ICH_HST_STA.  Return ERR_NO_ERROR\n");   
   // Clear status except for in_use semaphore
   OUTP(SMBusBaseAddr + ICH_HST_STA, 0xBF);
   return ERR_NO_ERROR;
}

// Checks SMBus host status register for completion of transaction
I2CSTATUS ICHTestSMBusStatus()
{
   int j;
   BYTE data;

// Wait for completion of the command, whether successful or not
// Timeout should not occur, but here for safety
   j = 0;
   while(! (INP(SMBusBaseAddr + ICH_HST_STA) & 0x3E))
   {
      if (j == MAX_DELAY)
	  {
		 debug_out(DETAILS_LVL,"ICHTestSMBusStatus.  Command failed.  Return ERR_TIMEOUT_HOST.\n"); 
         return ERR_TIMEOUT_HOST;
	  }
      else
      {
         DELAY(ICH_DELAY_TIME);
         j++;
      }
   }


// Wait for BUSY bit to become indicate 'not busy' (indicating that the SMBus 
// transaction has completed)
// This timeout really never should fail, but is included for safety.

// (08/15/2001) Removed.  Not really needed.
/*
   j = 0;
   while((data = INP(SMBusBaseAddr + ICH_HST_STA)) & 0x1)
   {
      if (j == MAX_DELAY)
         return ERR_TIMEOUT_HOST;
      else
      {
         DELAY(ICH_DELAY_TIME);
         j++;
      }
   }
*/

// (08/15/2001) Added to make up for last block removed.
	data = INP(SMBusBaseAddr + ICH_HST_STA);

// Test for successful completion.  If any of the 'fail' bits are set
// (bit 5 SMBALERT, bit 4 FAILED, bit 3 BUS_ERR, bit 2 DEV_ERR), 
//   return ERR_PIIX4SMB_ERROR
   if ( (data & 0x3c) != 0)
   {
//      sprintf(buffer, "status = %x", (data & 0x1c));
//      DbgPrint(buffer);
       debug_out(DETAILS_LVL,"ICHTestSMBusStatus.  Command failed.  Return ERR_ICHSMB_ERROR.\n");
       return ERR_ICHSMB_ERROR;
    }
    else
	{
       debug_out(DETAILS_LVL,"ICHTestSMBusStatus.  Command success.  Return ERR_NO_ERROR.\n");
       return ERR_NO_ERROR;
	}
}


// ---------------------------------------------------------------------------
// Reads one or more bytes of data from the I2C slave device
// using the Byte Data Read protocol on the ICH SMBus controller
I2CSTATUS I2CICHPointerSetThenRead2(BYTE IN addr, BYTE IN pointer,
                                      void FAR OUT *retval, int IN length, 
                                      int IN retry)
{
   BYTE value;
   short svalue;
   I2CSTATUS status;
   int i;

// Initialize retval
   for (i=0; i < length; i++)
      ((char FAR*)retval)[i] = 0;

   if (length == 2)
   {
   // Read Word
      status = ICHReadDataWord(addr, pointer, &svalue);
	  debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read WORD.  addr = %xh, pointer = %xh, svalue = %xh.\n", addr, pointer, svalue);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read WORD failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
            return status;
         else
         {
            DELAY(10000);
            return I2CICHPointerSetThenRead2(addr, pointer, retval,
               length, retry+1);
         }
      }
	  debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read WORD success. Word = %xh.\n", svalue);
      ((short FAR*)retval)[0] = svalue;
   }
   else
   {
   // Read 1 Byte
      status = ICHReadDataByte(addr, pointer, &value);
	  debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read BYTE.  addr = %xh, pointer = %xh, value = %xh.\n", addr, pointer, value);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"IBMSMB:i2cich:I2CICHPointerSetThenRead2.  Read BYTE failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
            return status;
         else
         {
            DELAY(10000);
            return I2CICHPointerSetThenRead2(addr, pointer, retval,
               length, retry+1);
         }
      }
	  debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read BYTE success. Byte = %xh.\n", value);
      ((char FAR *)retval)[0] = value;
      
   // Implement multi-byte read as a 'send command then read byte' followed
   // by multiple 'receive byte' operations.  This should work fine for 
   // EEPROMs and devices which implement 'auto increment' registers. 
      for (i = 1; i < length; i++)
      {
         status = ICHReceiveByte(addr, &value);
		 debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read multi bytes.  addr = %xh, value = %xh.\n", addr, value);
         if (status != ERR_NO_ERROR)
         {
			debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read multi bytes failed.  Retry.\n");
            if (retry >= MAX_RETRIES)
               return status;
            else
            {
               DELAY(10000);
               
            // Retry from the start because I'm not sure what might have happened
            // to the internal device address.  I think it is safer to reset the
            // internal device address with a ICHReadDataByte.   
               return I2CICHPointerSetThenRead2(addr, pointer, retval,
                  length, retry+1);
            }
         }
		 debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2.  Read multiple bytes success.  Byte = %xh.\n", value);
         ((char FAR *)retval)[i] = value;
      }
   }
   debug_out(DETAILS_LVL,"I2CICHPointerSetThenRead2. End func - ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
//   Writes one or more bytes of data from the I2C slave device
//   using the Byte Data Read protocol on the ICH SMBus controller
I2CSTATUS I2CICHPointerSetThenWrite2(BYTE IN addr, BYTE IN pointer,
                                       void FAR IN *value, int IN length, 
                                       int IN retry)
{
   I2CSTATUS status;
   int i, j;
//   char buffer[256];

   
   if (length == 2)
   {
	  debug_out(DETAILS_LVL,"I2CICHPointerSetThenWrite2.  Write WORD.  addr = %xh, pointer = %xh, value = %xh.\n", addr, pointer, ((short FAR *)value)[0]);
      status = ICHWriteDataWord(addr, pointer, ((short FAR *)value)[0]);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CICHPointerSetThenWrite2.  Write WORD failed.  Retry. \n"); 
         if (retry >= MAX_RETRIES)
            return status;
         else
         {
            DELAY(10000);
            return I2CICHPointerSetThenWrite2(addr, pointer, value,
               length, retry+1);
         }
      }
   }
   else
   {
	   for (i = 0; i < length; i++)
      {
		  debug_out(DETAILS_LVL,"I2CICHPointerSetThenWrite2.  Write BYTE.  addr = %xh, pointer = %xh, value = %xh\n", addr, pointer, ((char FAR *)value)[i]);
          status = ICHWriteDataByte(addr, pointer, ((char FAR *)value)[i]);
   
         j = 0;
         while (status != ERR_NO_ERROR)
         {
            j++;
   //         sprintf(buffer, "\t\tSMBus error for Write Data Byte on %1d try.\n", j);
   //         DbgPrint(buffer);
            if (j == MAX_RETRIES)
			{
				debug_out(DETAILS_LVL,"I2CICHPointerSetThenWrite2.  Write BYTE failed.  addr = %xh, pointer = %xh, value = %xh\n", addr, pointer, ((char FAR *)value)[i]);
                return status;
			}
            else
            {
               DELAY(10000);
               status = ICHWriteDataByte(addr, pointer, ((char FAR *)value)[i]);
            }
         }
        
		 debug_out(DETAILS_LVL,"I2CICHPointerSetThenWrite2.  Write BYTE successful.  addr = %xh, pointer = %xh, value = %xh\n", addr, pointer, ((char FAR *)value)[i]);
      // Increment the 'command' field.  This makes multi-byte writes 
      // with length > 2 work on EEPROMs and LMxx.  
         pointer++;
      }
   }
   debug_out(DETAILS_LVL,"I2CICHPointerSetThenWrite2.  End func - ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
//     Read one or more bytes from the ICH using the SMB
//     Receive Data protocol
I2CSTATUS I2CICHRead2(BYTE IN addr, void FAR OUT *retval, 
                        int IN length, int IN retry)
{
   BYTE value;
   I2CSTATUS status;
   int i, j;
//   char buffer[256];

// Initialize retval
   for (i=0; i<length; i++)
      ((char FAR*)retval)[i] = 0;

   for(i = 0; i < length; i++)
   {
      status = ICHReceiveByte(addr, &value);
      j = 0;
      while(status != ERR_NO_ERROR)
      {
         ++j;
//         sprintf(buffer, "\t\tSMBus error for Receive Byte on %1d try.\n", j);
//         DbgPrint(buffer);
         if(j == MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CICHRead2.  Read >= 1 bytes failed. addr = %xh, value = %xh.\n", addr, value);
            return status;
		 }
         else
         {
            DELAY(10000);
            status = ICHReceiveByte(addr, &value);
         }
      }
	  debug_out(DETAILS_LVL,"I2CICHRead2.  Read >= 1 bytes successful. addr = %xh, value = %xh.\n", addr, value);
      ((char FAR *)retval)[i] = value;
   }
   debug_out(DETAILS_LVL,"I2CICHRead2.  End func - ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
// Reads a block of data from the I2C slave device
// using the Block Data Read protocol on the ICH SMBus controller
I2CSTATUS I2CICHPointerSetThenReadBlock2(BYTE IN addr, BYTE IN pointer,
                                         BYTE FAR OUT *retval, int OUT *length, 
                                         int IN retry)
{
   I2CSTATUS status;
//  int i;  FH3

   status = ICHReadDataBlock1(addr, pointer, retval, length);

   return status;
}


// ---------------------------------------------------------------------------
I2CSTATUS I2CICHPointerSetThenWriteBlock2(BYTE IN addr, BYTE IN pointer,
                                          BYTE FAR IN *value, int IN length, 
                                          int IN retry)
{
   I2CSTATUS status;
//   int i, j;   FH3

   status = ICHWriteDataBlock1(addr, pointer, value, length);

   return status;
}


// ---------------------------------------------------------------------------
I2CSTATUS I2CICHPointerSet2(BYTE IN addr, BYTE IN command)
{
   I2CSTATUS status;
//   int i, j;   FH3

   status = ICHSendCommand(addr, command);

   return status;
}


// ---------------------------------------------------------------------------
// Implements the SMB Receive Byte protocol on the ICH
I2CSTATUS ICHReceiveByte(BYTE IN addr, BYTE FAR* OUT retval)
{
   BYTE data;
   I2CSTATUS status;

// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// then wait till bus is inactive
// ----------------------------------
   status = ICHWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

   debug_out(DETAILS_LVL,"ICHReceiveByte.  Configure ICH_XMIT_SLVA for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + ICH_XMIT_SLVA), (addr | SMBus_READ_Transaction));

// ----------------------------------
// Setup the SMB register for 1 or more Byte Read transactions
// wait for transaction to complete
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReceiveByte.  Configure ICH_HST_CNT - byte transaction.\n");
   data = 0x04;
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), data);
 
// start read operation
   debug_out(DETAILS_LVL,"ICHReceiveByte.  Configure ICH_HST_CNT - start transaction.\n");
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), (data | ICH_START));

// wait for completion
   status = ICHTestSMBusStatus();

// ----------------------------------
// Read value from the SMBHSTDAT0 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
	   *retval = INP(SMBusBaseAddr + ICH_HST_D0);
	   debug_out(DETAILS_LVL,"ICHReceiveByte.  Success - ICH_HST_D0 = %xh.\n", *retval);
   }

// ----------------------------------
// Clear SMBHSTSTS and return
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReceiveByte.  Clear ICH_HST_STA.\n");
   ICH_CLEAR_HOST_STATUS();

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Read protocol on the ICH SMBus controller
I2CSTATUS ICHReadDataByte(BYTE IN addr, BYTE IN pointer, BYTE FAR OUT *retval)
{
   BYTE data;
   I2CSTATUS status;
//   char buffer[256];

   status = ICHWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;
 
// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReadDataByte.  Configure ICH_XMIT_SLVA for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + ICH_XMIT_SLVA), (addr | SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReadDataByte.  Configure ICH_HST_CMD with offset. offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + ICH_HST_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Read operation
// not clear if start bit and command protocol bits can be set together
// so start transaction is done separately
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReadDataByte. Configure ICH_HST_CNT - byte data.\n");
   data = 0x08;        //  byte transaction
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), data);

// Start the transaction
   debug_out(DETAILS_LVL,"ICHReadDataByte. Configure ICH_HST_CNT - start transaction.\n");
   OUTP((SMBusBaseAddr + ICH_HST_CNT), (data | ICH_START));
   
   status = ICHTestSMBusStatus();
   
//    sprintf(buffer, "Status after SMBusTestStatus = %1x .", INP( (SMBusBaseAddr + SMBHSTSTS)));
//    DbgPrint(buffer);
 
// ----------------------------------
// Read returned values from the SMBHSTDAT0/1 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
	   *retval = INP(SMBusBaseAddr + ICH_HST_D0);
	   debug_out(DETAILS_LVL,"ICHReadDataByte. Success. ICH_HST_D0 = %xh.\n", *retval);
   }

   debug_out(DETAILS_LVL,"ICHReadDataByte. Clear ICH_HST_STA.\n");
   ICH_CLEAR_HOST_STATUS();

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Write protocol on the ICH SMBus controller
I2CSTATUS ICHWriteDataByte(BYTE IN addr, BYTE IN pointer, BYTE IN value)
{
   BYTE data;
   I2CSTATUS status;
//   char buffer[256];


   status = ICHWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHWriteDataByte.  Configure ICH_XMIT_SLVA for write to %xh.\n", addr);
   OUTP( (SMBusBaseAddr + ICH_XMIT_SLVA), (addr & ~SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHWriteDataByte.  Configure ICH_HST_CMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + ICH_HST_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Write operation and initiate transaction
// ----------------------------------
   debug_out(DETAILS_LVL,"In ICHWriteDataByte.  Configure ICH_HST_CNT - byte transaction.\n");
   data = 0x08;
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), data);
   debug_out(DETAILS_LVL,"ICHWriteDataByte.  Write %xh to ICH_HST_D0.\n", value);
   OUTP( (SMBusBaseAddr + ICH_HST_D0), value);
   
// Start the transaction
   debug_out(DETAILS_LVL,"ICHWriteDataByte.  Start write byte.\n");
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), (data | ICH_START));
   status = ICHTestSMBusStatus();

// ----------------------------------
// Clear ICH_HST_STA
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHWriteDataByte.  Clear ICH_HST_STA.\n");
   ICH_CLEAR_HOST_STATUS();
   
//    sprintf(buffer, "Status after TstsSMBusStatus = %1x .", INP( (SMBusBaseAddr + SMBHSTSTS)));
//    DbgPrint(buffer);
 
   return status;
}



// ---------------------------------------------------------------------------
//   Implements the Byte Data Read protocol on the ICH SMBus controller
I2CSTATUS ICHReadDataWord(BYTE IN addr, BYTE IN pointer, short FAR OUT *retval)
{
   BYTE data;
   I2CSTATUS status;
//   char buffer[256];

   status = ICHWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;
 
// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReadDataWord.  Configure ICH_XMIT_SLVA for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + ICH_XMIT_SLVA), (addr | SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReadDataWord.  Configure ICH_HST_CMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + ICH_HST_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Read operation
// not clear if start bit and command protocol bits can be set together
// so start transaction is done separately
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHReadDataWord.  Configure ICH_HST_CNT - word tansaction.\n");
   data = 0x0c;        //  word transaction
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), data);

// Start the transaction
   debug_out(DETAILS_LVL,"ICHReadDataWord.  Configure ICH_HST_CNT - start tansaction.\n");
   OUTP((SMBusBaseAddr + ICH_HST_CNT), (data | ICH_START));
   
   status = ICHTestSMBusStatus();
// ----------------------------------
// Read returned values from the ICH_HST_D0/1 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
      ((char FAR*)retval)[0] = INP(SMBusBaseAddr + ICH_HST_D0);
	   debug_out(DETAILS_LVL,"ICHReadDataWord. Success.  ICH_HST_D0 (lower 8bits of WORD) = %xh.\n", ((char FAR*)retval)[0]);
      ((char FAR*)retval)[1] = INP(SMBusBaseAddr + ICH_HST_D1);
	   debug_out(DETAILS_LVL,"ICHReadDataWord. Success.  ICH_HST_D1 (upper 8bits of WORD) = %xh.\n", ((char FAR*)retval)[1]);
   }

   debug_out(DETAILS_LVL,"ICHReadDataWord.  Clear ICH_HST_STA.\n");
   ICH_CLEAR_HOST_STATUS();

   return status;  
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Write protocol on the ICH SMBus controller
I2CSTATUS ICHWriteDataWord(BYTE IN addr, BYTE IN pointer, short IN value)
{
   BYTE data;
   I2CSTATUS status;
//   char buffer[256];


   status = ICHWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHWriteDataWord.  Configure ICH_XMIT_SLVA for write to %xh.\n", addr);
   OUTP( (SMBusBaseAddr + ICH_XMIT_SLVA), (addr & ~SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHWriteDataWord.  Configure ICH_HST_CMD with offset. offset = %xh\n", pointer);
   OUTP( (SMBusBaseAddr + ICH_HST_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Write operation and initiate transaction
// ----------------------------------
   debug_out(DETAILS_LVL,"CHWriteDataWord.  Configure ICH_HST_CNT- WORD transaction.\n");
   data = 0x0c;
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), data);
   debug_out(DETAILS_LVL,"ICHWriteDataWord.  Write %xh (lower 8bits of WORD) to ICH_HST_D0.\n", ((char FAR *)&value)[0]);
   OUTP( (SMBusBaseAddr + ICH_HST_D0), ((char FAR *)&value)[0]);
   debug_out(DETAILS_LVL,"ICHWriteDataWord.  Write %xh (upper 8bits of WORD) to ICH_HST_D1.\n", ((char FAR *)&value)[1]);
   OUTP( (SMBusBaseAddr + ICH_HST_D1), ((char FAR *)&value)[1]);
   
// Start the transaction
   debug_out(DETAILS_LVL,"ICHWriteDataWord.  Start word write.\n");
   OUTP( (SMBusBaseAddr + ICH_HST_CNT), (data | ICH_START));
   status = ICHTestSMBusStatus();

// ----------------------------------
// Clear SMBHSTSTS
// ----------------------------------
   debug_out(DETAILS_LVL,"ICHWriteDataWord.  Clear SMBus Host Status register.\n");
   ICH_CLEAR_HOST_STATUS();
   
//    sprintf(buffer, "Status after TstsSMBusStatus = %1x .", INP( (SMBusBaseAddr + SMBHSTSTS)));
//    DbgPrint(buffer);
 
   return status;
}


/* Reads one byte of data in block form into data[0] */
I2CSTATUS ICHReadDataBlock1 (BYTE IN address, BYTE IN command, BYTE OUT *data, int *length)
{
//   int i,mask,y,j;    FH3
   int i,j;	
//   long int timeout = 650;   FH3  
//   I2CSTATUS status;    FH3
   
   if(ICHWaitForSMBus())
   {
      return ERR_TIMEOUT;
   }

   // Setup the slave device address
   debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Configure ICH_XMIT_SLVA with device address = %xh.\n", address);
   OUTP(SMBusBaseAddr + ICH_XMIT_SLVA, (address));  // ||0x01 ???
  
   // Setup the offset to read from
   debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Configure ICH_HST_CMD with offset.  offset = %xh.\n", command);
   OUTP(SMBusBaseAddr + ICH_HST_CMD, command);
   
   // Zero the data register
   debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Clear ICH_HST_D0.\n");
   OUTP(SMBusBaseAddr + ICH_HST_D0, 0);
   
   // Read all the data
   i = 0;  // Number of bytes read (current)
   *length = 0;
   do
   {

	   //(08/15/2001) Heavy Modifications to BLOCK code
      // Issue Data block command
      if (i == (*length -1))
	  {
	      debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Configure ICH_HST_CNT - last byte.\n");
		  OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x20);  //Indicate last byte
	  }

	  if (i == 0)
	  {
		  debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Configure ICH_HST_CNT - start transaction.\n");
		  OUTP(SMBusBaseAddr + ICH_HST_CNT, (ICH_SMB_CMD_BLOCK | ICH_START));
      }
      else
      {
		  //An Note:The ICH_HST_STA register is defined a little differently for ICH2 and ICH4 this 0x80 might need to be changed.
		  debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Check ICH_HST_STA for byte done.\n");
		  OUTP(SMBusBaseAddr + ICH_HST_STA, 0x80);
      }

	  //(08/15/2001) Heavy Modifications to BLOCK code
      j = 0;
      while ( !(INP(SMBusBaseAddr + ICH_HST_STA) & 0x82))
	  {
		  if (INP(SMBusBaseAddr + ICH_HST_STA) & 0x1C)
		  {
			 // Return error code on SMBUS fail
			 OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
			 OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
			 debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Return ERR_BUS_ERROR.\n");
			 return ERR_BUS_ERROR;
		  }

		  j++;
		  if (j > MAX_DELAY)
			  break;
	  }

	  if (j > MAX_DELAY)
	  {
		 // Return error code on SMBUS fail
		 OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
		 OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
		 debug_out(DETAILS_LVL,"ICHReadDataBlock1.  Return ERR_NO_ACK.\n");
		 return ERR_NO_ACK;
	  }

      // Set length if not set yet
      if (i==0)
      {
         *length = INP(SMBusBaseAddr + ICH_HST_D0);
      }

      // Read next data byte
      data[i] = INP(SMBusBaseAddr + ICH_BLOCK_DB);

      i++;
   } while (i < *length);

   //(08/15/2001) Clear the control register.
   OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);

   // Clear the status
   OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
   debug_out(DETAILS_LVL,"ICHReadDataBlock1.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}



/* Writes one byte of data in block form from data */
I2CSTATUS ICHWriteDataBlock1 (BYTE address, BYTE command, BYTE *data, int length)
{
//   int i,mask,j;    FH3
   int i,j;
	
//   I2CSTATUS status;   FH3
   
   // put an upper and lower boundary on count
   if (length > 32)
      length = 32;
   if (length < 0)
      length = 0;

   if(ICHWaitForSMBus())
   {
      return ERR_TIMEOUT;                                
   }
   
   // Set up the slave device address
   //An note: The 0xFE mask does not make sense.
   debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Configure ICH_XMIT_SLVA for write to %xh.\n", address);
   OUTP(SMBusBaseAddr + ICH_XMIT_SLVA, (address & 0xFE));

   // Setup the write command
   debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Configure ICH_HST_CMD with offset.  offset = %xh.\n", command);
   OUTP(SMBusBaseAddr + ICH_HST_CMD, command);

   // Setup the number of bytes to write as 1
   debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Configure ICH_HST_D0 with number of bytes to write = %dd.\n", length);
   OUTP(SMBusBaseAddr + ICH_HST_D0, length); 

   // Send all the data
   i = 0;
   while (i < length)
   {
      OUTP(SMBusBaseAddr + ICH_BLOCK_DB, data[i]);

	  //(08/15/2001) Heavy Modifications to BLOCK code
   
      // Issue Data block command
      // NOTE: Fixed defect 11261 - 12/06/00. This defect was caused by the
      //        repeated issuing of the ICH_START command which effectively locked
      //        out all other access.
      if (i == 0) 
          OUTP(SMBusBaseAddr + ICH_HST_CNT, (ICH_SMB_CMD_BLOCK  | ICH_START));
      else
          OUTP(SMBusBaseAddr + ICH_HST_STA,  0x80);

      // Wait for status bit to be set or error
      j = 0;
      while ( !(INP(SMBusBaseAddr + ICH_HST_STA) & 0x82))
	  {
		  if (INP(SMBusBaseAddr + ICH_HST_STA) & 0x1C)
		  {
			 // Return error code on SMBUS fail
			 OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
			 OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
			 debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Return ERR_BUS_ERROR.\n");
			 return ERR_BUS_ERROR;
		  }

		  j++;
		  if (j > MAX_DELAY)
			  break;
	  }

	   //(08/15/2001) Heavy Modifications to BLOCK code
   	  if (j > MAX_DELAY)
	  {
		 // Return error code on SMBUS fail
		 OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
		 OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
		 debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Return ERR_NO_ACK.\n"); 
		 return ERR_NO_ACK;
	  }

      i++;
   }

   //(08/15/2001) Clear the control register.
   debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Clear ICH_HST_CNT.\n");
   OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
   
   // Clear the status
   debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  Clear ICH_HST_STA.\n");
   OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));

   debug_out(DETAILS_LVL,"ICHWriteDataBlock1.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}



/* Sends a command */
I2CSTATUS ICHSendCommand (BYTE IN address, BYTE IN command)
{
//   int error,i,j,mask;   FH3
   int j;
//   I2CSTATUS status;     FH3

   if(ICHWaitForSMBus())
   {
      return ERR_TIMEOUT;                                
   }
 
   // Setup the slave device address
   debug_out(DETAILS_LVL,"ICHSendCommand.  Configure ICH_XMIT_SLVA with device address.  address = %xh.\n", address);
   OUTP(SMBusBaseAddr + ICH_XMIT_SLVA, (address));

   // Setup the command to send
   debug_out(DETAILS_LVL,"ICHSendCommand.  Configure ICH_HST_CMD with offset.  offset = %xh.\n", command);
   OUTP(SMBusBaseAddr + ICH_HST_CMD, command);
   
   debug_out(DETAILS_LVL,"ICHSendCommand.  Configure ICH_HST_CNT - byte transaction and start transaction.\n");
   OUTP(SMBusBaseAddr + ICH_HST_CNT, (ICH_SMB_CMD_BYTE | ICH_START));

   DELAY(DELAY_FOR_IO);

	  //(08/15/2001) Heavy Modifications to SendCommand code
   j = 0;
//   status = ICHTestSMBusStatus();
	  while ( !(INP(SMBusBaseAddr + ICH_HST_STA) & 0x82))
	  {
		  if (INP(SMBusBaseAddr + ICH_HST_STA) & 0x1C)
		  {
			 // Return error code on SMBUS fail
			 OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
			 OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
			 debug_out(DETAILS_LVL,"ICHSendCommand.  Return ERR_BUS_ERROR.\n");
			 return ERR_BUS_ERROR;
		  }

		  j++;
		  if (j > MAX_DELAY)
			  break;
	  }

	  if (j > MAX_DELAY)
	  {
		 // Return error code on SMBUS fail
		 OUTP(SMBusBaseAddr + ICH_HST_CNT, 0x00);
		 OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
		 debug_out(DETAILS_LVL,"ICHSendCommand.  Return ERR_NO_ACK.\n"); 
		 return ERR_NO_ACK;
	  }

   // Clear the status
   debug_out(DETAILS_LVL,"ICHSendCommand.  Clear ICH_HST_STA.\n");  
   OUTP(SMBusBaseAddr + ICH_HST_STA, INP(SMBusBaseAddr + ICH_HST_STA));
   
   debug_out(DETAILS_LVL,"ICHSendCommand.  End function success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


