
// **********************************************************************
// *                                                                    *
// * Common code file for the SMBUS on the SIS Chip.                    *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

#include "i2cos.h"
#include "i2cio.h"
#include "i2csis.h"
#include "pci.h"
// #include <string.h>   FH3

#define SIS_DELAY_TIME 100
 
extern int MAX_RETRIES;

// ------------------------------
//  Internal Function Prototypes
// ------------------------------
I2CSTATUS SISWaitAndClearSMBus(void);
I2CSTATUS SISWaitForCompletion(void);
I2CSTATUS SISReceiveByte(BYTE slave_addr, BYTE FAR *byte_read);
I2CSTATUS SISSendByte(BYTE slave_addr, BYTE byte_written);
I2CSTATUS SISReadDataByte(BYTE slave_addr, BYTE command_code, BYTE FAR *byte_read);
I2CSTATUS SISWriteDataByte(BYTE slave_addr, BYTE command_code, BYTE byte_written);
I2CSTATUS SISReadDataWord(BYTE IN addr, BYTE IN pointer, short FAR OUT *retval);
I2CSTATUS SISWriteDataWord(BYTE IN addr, BYTE IN pointer, short IN value);

//-------------------
// External global variables
//-------------------
extern unsigned int SMBusBaseAddr;
extern int IsSISFound;    // defined in i2cio.c


// ---------------------------------------------------------------------------


// Wait until neither the host or slave busy bits are set.
// This timeout really never should fail, but is included for safety.
// Returns ERR_NO_ERROR on success.
I2CSTATUS SISWaitAndClearSMBus()
{  
   int j = 0;

   debug_out(DETAILS_LVL,"SISWaitAndClearSMBus.  Wait for SMBus not busy\n");
   while ( (INP(SMBusBaseAddr + SIS_SMB_CNT) & SIS_HOST_BUSY) || 
           (INP(SMBusBaseAddr + SIS_SMB_CNT) & SIS_SLV_BUSY) )
   {
      if (j == MAX_DELAY)
	  {
		 debug_out(DETAILS_LVL,"SISWaitAndClearSMBus.  Return ERR_TIMEOUT.\n");
         return ERR_TIMEOUT;
	  }
      else
      {
         DELAY(SIS_DELAY_TIME);
         j++;
      }
   }
   
   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISWaitAndClearSMBus.  Host registers cleared.  Return ERR_NO_ERROR.\n");
   
   return ERR_NO_ERROR;
}


// Checks SMBus host status register for completion of transaction
I2CSTATUS SISWaitForCompletion()
{
   int j;
   BYTE data;

   j = 0;
    debug_out(DETAILS_LVL,"SISWaitForCompletion.  Wait for completion.\n");
	while((data = INP(SMBusBaseAddr + SIS_SMB_CNT)) & (SIS_HOST_BUSY | SIS_SLV_BUSY)) 
   {                                        
      if (j == MAX_DELAY)
	  {
		 debug_out(DETAILS_LVL,"SISWaitForCompletion.  Transaction failed.  Return ERR_TIMEOUT.\n");
         return ERR_TIMEOUT;
	  }
      else
      {
         DELAY(SIS_DELAY_TIME);
         j++;
      }
   }

	data = INP(SMBusBaseAddr + SIS_SMB_STS);
   if ( (data & 0x08) != 0x08)
   {
	   debug_out(DETAILS_LVL,"SISWaitForCompletion.  Error occurred.  Return ERR_SISSMB_ERROR.\n");
       return ERR_SISSMB_ERROR;
    }

	data = INP(SMBusBaseAddr + SIS_SMB_STS);
   if ( (data & 0xe6) != 0)
   {
	   debug_out(DETAILS_LVL,"SISWaitForCompletion.  Error occurred.  Return ERR_SISSMB_ERROR.\n");
       return ERR_SISSMB_ERROR;
    }
    else
	{
	   debug_out(DETAILS_LVL,"SISWaitForCompletion.  End func - success.  Return ERR_NO_ERROR.\n");
       return ERR_NO_ERROR;
	}
}


// ---------------------------------------------------------------------------
// Reads one or more bytes of data from the I2C slave device
// using the Byte Data Read protocol on the SIS SMBus controller
I2CSTATUS I2CSISPointerSetThenRead2(BYTE IN addr, BYTE IN pointer,
                                      void FAR OUT *retval, int IN length, 
                                      int IN retry)
{
   BYTE value;
   short svalue;
   I2CSTATUS status;
   int i;
//   char buffer[256];

// Initialize retval
   for (i=0; i < length; i++)
      ((char FAR*)retval)[i] = 0;

	// DbgPrint("SIS - In Pointer Set Then Read");   FH3
    debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2... \n");
   

   if (length == 2)
   {
   // Read Word
      status = SISReadDataWord(addr, pointer, &svalue);
      debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Read WORD.  addr = %xh, pointer = %xh, svalue = %xh.\n", addr, pointer, svalue);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Read WORD failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
		 {
             debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  All retries failed.\n");
			 return status;
		 }
         else
         {
            DELAY(10000);
            return I2CSISPointerSetThenRead2(addr, pointer, retval,
               length, retry+1);
         }
      }
	  debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Data read back = %xh.\n", svalue);
      ((short FAR*)retval)[0] = svalue;
   }
   else
   {
   // Read 1 Byte
      status = SISReadDataByte(addr, pointer, &value);
      debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Read BYTE.  addr =%xh, pointer = %xh, value = %xh.\n", addr, pointer, value);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Read BYTE failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
		 {
             debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  All retries failed.\n");
			 return status;
		 }
         else
         {
            DELAY(10000);
            return I2CSISPointerSetThenRead2(addr, pointer, retval,
               length, retry+1);
         }
      }
	  debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Data read back = %xh.\n", value);
      ((char FAR *)retval)[0] = value;
      
   // Implement multi-byte read as a 'send command then read byte' followed
   // by multiple 'receive byte' operations.  This should work fine for 
   // EEPROMs and devices which implement 'auto increment' registers. 
      for (i = 1; i < length; i++)
      {
         status = SISReceiveByte(addr, &value);
         debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Read multi bytes.  addr = %xh, value = %xh.\n", addr, value);
         if (status != ERR_NO_ERROR)
         {
			debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Reading multi bytes failed.  Retry.\n");
            if (retry >= MAX_RETRIES)
			{
			   debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  All retries failed.\n");
               return status;
			}
            else
            {
               DELAY(10000);
               
            // Retry from the start because I'm not sure what might have happened
            // to the internal device address.  I think it is safer to reset the
            // internal device address with a SISReadDataByte.   
               return I2CSISPointerSetThenRead2(addr, pointer, retval,
                  length, retry+1);
            }
         }
		 debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  Data read back = %xh.\n", value);
         ((char FAR *)retval)[i] = value;
      }
   }
   
   debug_out(DETAILS_LVL,"I2CSISPointerSetThenRead2.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
//   Writes one or more bytes of data from the I2C slave device
//   using the Byte Data Read protocol on the SIS SMBus controller
I2CSTATUS I2CSISPointerSetThenWrite2(BYTE IN addr, BYTE IN pointer,
                                       void FAR IN *value, int IN length, 
                                       int IN retry)
{
   I2CSTATUS status;
   int i, j;
//   char buffer[256];

   
   if (length == 2)
   {
      status = SISWriteDataWord(addr, pointer, ((short FAR *)value)[0]);
	  debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  Write WORD.  addr = %xh, pointer = %xh, value to write = %xh.\n", addr, pointer, ((short FAR *)value)[0]);
      DELAY(20000);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  Write WORD failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  All retries failed.\n");
            return status;
		 }
         else
         {
            DELAY(10000);
            return I2CSISPointerSetThenWrite2(addr, pointer, value,
               length, retry+1);
         }
      }
   }
   else
   {
      for (i = 0; i < length; i++)
      {
         status = SISWriteDataByte(addr, pointer, ((char FAR *)value)[i]);
		 debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  Write BYTE.  addr = %xh, pointer = %xh, value to write = %xh.\n", addr, pointer, ((char FAR *)value)[i]);
         j = 0;
         while (status != ERR_NO_ERROR)
         {
			debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  Writing BYTE failed.  Retry.\n");
            j++;
   //         sprintf(buffer, "\t\tSMBus error for Write Data Byte on %1d try.\n", j);
   //         DbgPrint(buffer);
            if (j == MAX_RETRIES)
			{
			   debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  All retries failed.\n");
               return status;
			}
            else
            {
               DELAY(10000);
               status = SISWriteDataByte(addr, pointer, ((char FAR *)value)[i]);
            }
         }
         
      // Increment the 'command' field.  This makes multi-byte writes 
      // with length > 2 work on EEPROMs and LMxx.  
         pointer++;
      }
   }
   debug_out(DETAILS_LVL,"I2CSISPointerSetThenWrite2.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
//     Read one or more bytes from the SIS using the SMB
//     Receive Data protocol
I2CSTATUS I2CSISRead2(BYTE IN addr, void FAR OUT *retval, 
                        int IN length, int IN retry)
{
   BYTE value;
   I2CSTATUS status;
   int i, j;
//   char buffer[256];

	// DbgPrint("SIS - Read2");   FH3
    debug_out(DETAILS_LVL,"I2CSISRead2...\n");
	

// Initialize retval
   for (i=0; i<length; i++)
      ((char FAR*)retval)[i] = 0;

   for(i = 0; i < length; i++)
   {
      status = SISReceiveByte(addr, &value);
	  debug_out(DETAILS_LVL,"I2CSISRead2.  Read BYTE from %xh. value = %xh.\n", addr, value);
      j = 0;
      while(status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CSISRead2.  Reading BYTE failed.  Retry.\n");
         ++j;
//         sprintf(buffer, "\t\tSMBus error for Receive Byte on %1d try.\n", j);
//         DbgPrint(buffer);
         if(j == MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CSISRead2.  All retries failed.\n");
            return status;
		 }
         else
         {
            DELAY(10000);
            status = SISReceiveByte(addr, &value);
         }
      }
	  debug_out(DETAILS_LVL,"I2CSISRead2.  Data read = %xh.\n", value);
      ((char FAR *)retval)[i] = value;
   }
   debug_out(DETAILS_LVL,"I2CSISRead2.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
// Implements the SMB Receive Byte protocol on the SIS
I2CSTATUS SISReceiveByte(BYTE IN addr, BYTE FAR* OUT retval)
{
   BYTE data;
//   int i;
   I2CSTATUS status;
//   char buffer[256];


	// DbgPrint("SIS - ReceiveByte");   FH3
    debug_out(DETAILS_LVL,"SISReceiveByte...\n");
 	
	

// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// then wait till bus is inactive
// ----------------------------------
   status = SISWaitAndClearSMBus();
   if (status != ERR_NO_ERROR)
      return status;

   debug_out(DETAILS_LVL,"SISReceiveByte.  Configure SIS_SMB_ADDR for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + SIS_SMB_ADDR), (addr | SMBus_READ_Transaction));

// ----------------------------------
// Setup the SMB register for 1 or more Byte Read transactions
// wait for transaction to complete
// ----------------------------------
   debug_out(DETAILS_LVL,"SISReceiveByte.  Configure SIS_SMBHOST_CNT for byte transaction.\n");
   data = 0x01;
   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), data);

// start read operation
   debug_out(DETAILS_LVL,"SISReceiveByte.  Start transaction.\n");
   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), (data | SIS_START));
   
// wait for completion
   status = SISWaitForCompletion();

// ----------------------------------
// Read value from the SIS_SMB_BYTE0 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
	   *retval = INP(SMBusBaseAddr + SIS_SMB_BYTE0);
	   debug_out(DETAILS_LVL,"SISReceiveByte. Data read back = %xh.\n", *retval);
   }


// ----------------------------------
// Clear SISSMBHSTSTA and return
// ----------------------------------
   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISReceiveByte.  SISSMBHSTSTA cleared.\n");

//    sprintf(buffer, "Status after ReceiveByteStatus = %2x .\n", INP( (SMBusBaseAddr + SISSMBHSTSTA)));
    //  myPrintf(buffer);
 
   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Read protocol on the SIS SMBus controller
I2CSTATUS SISReadDataByte(BYTE IN addr, BYTE IN pointer, BYTE FAR OUT *retval)
{
   BYTE data;
   I2CSTATUS status;

   status = SISWaitAndClearSMBus();
   if (status != ERR_NO_ERROR)
      return status;

 
// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// ----------------------------------
   debug_out(DETAILS_LVL,"SISReadDataByte.  Configure SIS_SMB_ADDR for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + SIS_SMB_ADDR), (addr | SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"SISReadDataByte.  Configure SIS_SMB_CMD with offset.  offset = %xh\n", pointer);
   OUTP( (SMBusBaseAddr + SIS_SMB_CMD), pointer);
	

// ----------------------------------
// Setup the SMB register for a Byte Data Read operation
// ----------------------------------
   data = 0x02;        //  byte transaction
   debug_out(DETAILS_LVL,"SISReadDataByte.  Configure SIS_SMBHOST_CNT - byte.\n");
   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), data);


// Start the transaction
   debug_out(DETAILS_LVL,"SISReadDataByte.  Start transaction.\n");
   OUTP((SMBusBaseAddr + SIS_SMBHOST_CNT), (data | SIS_START));
   
   status = SISWaitForCompletion();
   
 
// ----------------------------------
// Read returned values from the SIS_SMB_BYTE0/1 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
      *retval = INP(SMBusBaseAddr + SIS_SMB_BYTE0);
	   debug_out(DETAILS_LVL,"SISReadDataByte.  Data read back = %xh.\n", *retval);
   }


   DELAY(SIS_DELAY_TIME);
   
   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISReadDataByte.  Host registers cleared.\n");

   DELAY(SIS_DELAY_TIME);

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Write protocol on the SIS SMBus controller
I2CSTATUS SISWriteDataByte(BYTE IN addr, BYTE IN pointer, BYTE IN value)
{
   BYTE data;   
   I2CSTATUS status;
   
   status = SISWaitAndClearSMBus();
   if (status != ERR_NO_ERROR)
      return status;

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   debug_out(DETAILS_LVL,"SISWriteDataByte.  Configure SIS_SMB_ADDR for write to %xh.\n", addr);
   OUTP( (SMBusBaseAddr + SIS_SMB_ADDR), (addr & ~SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"SISWriteDataByte.  Configure SIS_SMB_CMD with offset.  offset = %xh\n", pointer);
   OUTP( (SMBusBaseAddr + SIS_SMB_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Write operation and initiate transaction
// ----------------------------------
   data = 0x02;
   debug_out(DETAILS_LVL,"SISWriteDataByte.  Write %xh to SIS_SMB_BYTE0.\n", value);
   OUTP( (SMBusBaseAddr + SIS_SMB_BYTE0), value);

// Start the transaction
   debug_out(DETAILS_LVL,"SISWriteDataByte.  Configure SIS_SMBHOST_CNT - byte data transaction and start transaction.\n");
   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), (data | SIS_START)); 
   
   status = SISWaitForCompletion();

// ----------------------------------
// Clear SISSMBHSTSTA
// ----------------------------------

   DELAY(SIS_DELAY_TIME);

   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISWriteDataByte.  Host registers cleared.\n");

   DELAY(SIS_DELAY_TIME);
 
   return status;
}


// ---------------------------------------------------------------------------
//     Implements the SMB Send Byte protocol on the SIS
I2CSTATUS SISSendByte(BYTE IN addr, BYTE IN value)
{
   BYTE data;
//      int i;

   I2CSTATUS status;
//   char buffer[256];

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   status = SISWaitAndClearSMBus();
   if (status != ERR_NO_ERROR)
      return status;

   debug_out(DETAILS_LVL,"SISSendByte.  Configure SIS_SMB_ADDR for write to %xh.\n", addr);
   OUTP( (SMBusBaseAddr + SIS_SMB_ADDR), (addr & ~SMBus_READ_Transaction));

// ----------------------------------
// Setup the SMB register for 1 or more Byte Read transactions
// ----------------------------------
   data = 0x01;
//   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), data);

   debug_out(DETAILS_LVL,"SISSendByte.  Write %xh to SIS_SMB_BYTE0.\n", value);
   OUTP( (SMBusBaseAddr + SIS_SMB_BYTE0), value);

// Start write operation
   debug_out(DETAILS_LVL,"SISSendByte.  Configure SIS_SMBHOST_CNT - byte transaction and start transaction.\n");
   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), (data | SIS_START));

// Wait for completion
   status = SISWaitForCompletion();
   
   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISSendByte.  Host registers cleared.\n");

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Read protocol on the SIS SMBus controller
I2CSTATUS SISReadDataWord(BYTE IN addr, BYTE IN pointer, short FAR OUT *retval)
{
   BYTE data;
//      int i;

   I2CSTATUS status;
//   char buffer[256];

   status = SISWaitAndClearSMBus();
   if (status != ERR_NO_ERROR)
      return status;
 
// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// ----------------------------------
   debug_out(DETAILS_LVL,"SISReadDataWord.  Configure SIS_SMB_ADDR for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + SIS_SMB_ADDR), (addr | SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"SISReadDataWord.  Configure SIS_SMB_CMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + SIS_SMB_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Read operation
// not clear if start bit and command protocol bits can be set together
// so start transaction is done separately
// ----------------------------------
   data = 0x03;        //  word transaction
//   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), data);

// Start the transaction
   debug_out(DETAILS_LVL,"SISReadDataWord.  Configure SIS_SMBHOST_CNT - word data transaction and start transaction.\n");
   OUTP((SMBusBaseAddr + SIS_SMBHOST_CNT), (data | SIS_START));
   
   status = SISWaitForCompletion();
   
//    sprintf(buffer, "Status after ReadWordStatus = %2x .\n", INP( (SMBusBaseAddr + SISSMBHSTSTA)));
    //  myPrintf(buffer);
 
// ----------------------------------
// Read returned values from the SIS_SMB_BYTE0/1 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
      ((char FAR*)retval)[0] = INP(SMBusBaseAddr + SIS_SMB_BYTE0);
	   debug_out(DETAILS_LVL,"SISReadDataWord.  SIS_SMB_BYTE0 (lower 8bits of WORD) = %xh.\n", ((char FAR*)retval)[0]);
      ((char FAR*)retval)[1] = INP(SMBusBaseAddr + SIS_SMB_BYTE1);
	   debug_out(DETAILS_LVL,"SISReadDataWord.  SIS_SMB_BYTE1 (upper 8bits of WORD) = %xh.\n", ((char FAR*)retval)[1]);
   }
   
   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISReadDataWord.  Host regsiters cleared.\n");

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Write protocol on the SIS SMBus controller
I2CSTATUS SISWriteDataWord(BYTE IN addr, BYTE IN pointer, short IN value)
{
   BYTE data;
   I2CSTATUS status;
//   char buffer[256];
//      int i;


   status = SISWaitAndClearSMBus();
   if (status != ERR_NO_ERROR)
      return status;

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   debug_out(DETAILS_LVL,"SISWriteDataWord.  Configure SIS_SMB_ADDR for write from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + SIS_SMB_ADDR), (addr & ~SMBus_READ_Transaction));

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"SISWriteDataWord.  Configure SIS_SMB_CMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + SIS_SMB_CMD), pointer);

// ----------------------------------
// Setup the SMB register for a Byte Data Write operation and initiate transaction
// ----------------------------------
   data = 0x03;
//   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), data);

   debug_out(DETAILS_LVL,"SISWriteDataWord.  Write %xh (lower 8bits) into SIS_SMB_BYTE0.\n", ((char FAR *)&value)[0]);
   OUTP( (SMBusBaseAddr + SIS_SMB_BYTE0), ((char FAR *)&value)[0]);

   debug_out(DETAILS_LVL,"SISWriteDataWord.  Write %xh (upper 8bits) into SIS_SMB_BYTE1.\n", ((char FAR *)&value)[1]);
   OUTP( (SMBusBaseAddr + SIS_SMB_BYTE1), ((char FAR *)&value)[1]);

// Start the transaction
   debug_out(DETAILS_LVL,"SISWriteDataWord.  Configure SIS_SMBHOST_CNT - word data and start transaction.\n");
   OUTP( (SMBusBaseAddr + SIS_SMBHOST_CNT), (data | SIS_START));
   
   status = SISWaitForCompletion();

// ----------------------------------
// Clear SISSMBHSTSTA
// ----------------------------------
   SIS_CLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"SISWriteDataWord.  Host regsiters cleared.\n");
   
   return status;
}
