
// **********************************************************************
// *                                                                    *
// * Common code file for the SMBUS on the VIA Chip and ICH chip        *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

/* *****************************************************************
  Description:
     Portable code to perform actual communication with the
     I2C using the VIA Chip and ICH chip.
     ICH register operation is the same as VIA except for status which
     implements an SMBALERT.  If this line is not used by system then the 
     VIA functions will work.  Otherwise must also use i2cich.c to build
******************************************************************* */

#include "i2cos.h"
#include "i2cio.h"
#include "i2cvia.h"
#include "pci.h"
// #include <string.h>   FH3
// #include <conio.h>    FH3
 
int VIADELAY = 20000;  // this is minimum for good write operation

extern int MAX_RETRIES;

// ------------------------------
//  Internal Function Prototypes
// ------------------------------
I2CSTATUS VIAWaitForSMBus(void);
I2CSTATUS VIABlockWaitForSMBus(void);
I2CSTATUS VIATestSMBusStatus(void);
I2CSTATUS VIAReceiveByte(BYTE slave_addr, BYTE FAR *byte_read);
I2CSTATUS VIASendByte(BYTE slave_addr, BYTE byte_written);
I2CSTATUS VIAReadDataByte(BYTE slave_addr, BYTE command_code, BYTE FAR *byte_read);
I2CSTATUS VIAWriteDataByte(BYTE slave_addr, BYTE command_code, BYTE byte_written);
I2CSTATUS VIAReadDataWord(BYTE IN addr, BYTE IN pointer, short FAR OUT *retval);
I2CSTATUS VIAWriteDataWord(BYTE IN addr, BYTE IN pointer, short IN value);
I2CSTATUS VIAReadDataBlock1(BYTE IN address, BYTE IN command, BYTE OUT *data, int *length);
I2CSTATUS VIAWriteDataBlock1(BYTE IN address, BYTE IN command, BYTE IN *data, int length);
I2CSTATUS VIASendCommand(BYTE IN address, BYTE IN command);

//-------------------
// External global variables
//-------------------
extern unsigned int SMBusBaseAddr;
extern int IsVIAFound;    // defined in i2cio.c

// ---------------------------------------------------------------------------


// Wait until neither the host or slave busy bits are set.
// This timeout really never should fail, but is included for safety.
// Returns ERR_NO_ERROR on success.
I2CSTATUS VIAWaitForSMBus()
{  
   int j;
        
   j = 0;
   debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Wait for SMBus not busy.\n");
   while ( (INP(SMBusBaseAddr + VIASMBHSTSTS) & VIASMB_HOST_BUSY) || 
           (INP(SMBusBaseAddr + VIASMBSLVSTS) & VIASMB_SLV_BUSY) )
   {
      if (j == MAX_DELAY)
	  {
		 VIAKILL_TRANSACTION();
		 debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Transaction killed.  Return ERR_TIMEOUT.\n");
         return ERR_TIMEOUT;
	  }
      else
      {
         DELAY(DELAY_TIME);
         j++;
      }
   }
   debug_out(DETAILS_LVL,"VIAWaitForSMBus.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}

// Wait until neither the host or slave busy bits are set.
// This timeout really never should fail, but is included for safety.
// Returns ERR_NO_ERROR on success.
I2CSTATUS VIABlockWaitForSMBus()
{  
   int j;
        
   j = 0;
   debug_out(DETAILS_LVL,"VIABlockWaitForSMBus.  Wait for SMBus not busy.\n");
   while ( (INP(SMBusBaseAddr + VIASMBHSTSTS) & VIASMB_HOST_BUSY) || 
           (INP(SMBusBaseAddr + VIASMBSLVSTS) & VIASMB_SLV_BUSY) )
   {
      if (j == MAX_DELAY)
	  {
		 VIAKILL_TRANSACTION();
		 debug_out(DETAILS_LVL,"VIABlockWaitForSMBus.  Transaction killed.  Return ERR_TIMEOUT.\n"); 
         return ERR_TIMEOUT;
	  }
      else
      {
         BLOCKDELAY(DELAY_TIME);
         j++;
      }
   }
   debug_out(DETAILS_LVL,"VIABlockWaitForSMBus.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// Checks SMBus host status register for completion of transaction
I2CSTATUS VIATestSMBusStatus()
{
   int j;
   BYTE data;
//   char buffer[256];

   //  add delay for Via chipset
   if (IsVIAFound)
   {
	  debug_out(DETAILS_LVL,"VIATestSMBusStatus.  VIA found.  Delaying for VIA.\n");
      DELAY(VIADELAY);
   }
   else
   {


// Wait for BUSY bit to become 'busy' (indicating that the SMBus 
// transaction has started)
// This timeout really never should fail, but is included for safety.
//
// Note: 
// It is unsafe to wait for the busy bit to become active since this
// is timing dependent.  This would likely fail if the CPU was 
// throttled down by APM or ACPI.  Instead, we will wait for either
// the 'busy bit' to become set or the 'successful completion' bit to be set.
// If the transaction completes successfully while the processor is 
// throttled down, the following loop should exit on the 'successful
// completion (INTER) bit', and the following test should be 'not busy',
// so the function should proceed normally.
      j = 0;
	  debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Wait for completion.\n");
      while(! (INP(SMBusBaseAddr + VIASMBHSTSTS) & 0x3))
      {
         if (j == MAX_DELAY)
		 {
		    VIAKILL_TRANSACTION();
			debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Transaction killed.  Return ERR_TIMEOUT.\n");
			return ERR_TIMEOUT;
		 }
         else
         {
            DELAY(DELAY_TIME);
            j++;
         }
      } 
   }  // end IsVIAFound if.. else
         
// Wait for BUSY bit to become 'not busy' (indicating that the SMBus 
// transaction has completed)
// This timeout really never should fail, but is included for safety.
   j = 0;
   debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Wait for completion.\n");
   while((data = INP(SMBusBaseAddr + VIASMBHSTSTS)) & 0x1)
   {
      if (j == MAX_DELAY)
	  { 
		 VIAKILL_TRANSACTION();
		 debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Transaction killed.  Return ERR_TIMEOUT.\n");
		 return ERR_TIMEOUT;
	  }
      else
      {
         DELAY(DELAY_TIME);
         j++;
      }
   }

// Test for successful completion.  If any of the 'fail' bits are set 
// (bit 4 FAILED, bit 3 BUS_ERR, bit 2 DEV_ERR), return ERR_VIASMB_ERROR 
   if ( (data & 0x1c) != 0)
   {
//      sprintf(buffer, "status = %x", (data & 0x1c));
//      DbgPrint(buffer);
       debug_out(DETAILS_LVL,"VIAWaitForSMBus.  Transaction failed.  Return ERR_VIASMB_ERROR.\n");
       return ERR_VIASMB_ERROR;
    }
    else
	{
	   debug_out(DETAILS_LVL,"VIAWaitForSMBus.  End func - success.  Return ERR_NO_ERROR.\n");
       return ERR_NO_ERROR;
	}
}


// ---------------------------------------------------------------------------
// Reads one or more bytes of data from the I2C slave device
// using the Byte Data Read protocol on the VIA SMBus controller
I2CSTATUS I2CVIAPointerSetThenRead2(BYTE IN addr, BYTE IN pointer,
                                      void FAR OUT *retval, int IN length, 
                                      int IN retry)
{
   BYTE value;
   short svalue;
   I2CSTATUS status;
   int i;
//   char buffer[256];

// Initialize retval
   for (i=0; i < length; i++)
      ((char FAR*)retval)[i] = 0;

   if (length == 2)
   {
   // Read Word
      status = VIAReadDataWord(addr, pointer, &svalue);
      debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Read WORD.  addr = %xh, pointer = %xh, svalue = %xh.\n", addr, pointer, svalue);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Read WORD failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CVIAPointerSetThenReade2.  All retries failed.\n");
            return status;
		 }
         else
         {
            DELAY(10000);
            return I2CVIAPointerSetThenRead2(addr, pointer, retval,
               length, retry+1);
         }
      }
	  debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Value read back = %xh.\n", svalue);
      ((short FAR*)retval)[0] = svalue;
   }
   else
   {
   // Read 1 Byte
      status = VIAReadDataByte(addr, pointer, &value);
      debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Read BYTE.  addr = %xh, pointer = %xh, value = %xh.\n", addr, pointer, value);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Read BYTE failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  All retries failed.\n");
            return status;
		 }
         else
         {
            DELAY(10000);
            return I2CVIAPointerSetThenRead2(addr, pointer, retval,
               length, retry+1);
         }
      }
	  debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Value read back  = %xh.\n", value);
      ((char FAR *)retval)[0] = value;
      
   // Implement multi-byte read as a 'send command then read byte' followed
   // by multiple 'receive byte' operations.  This should work fine for 
   // EEPROMs and devices which implement 'auto increment' registers. 
      for (i = 1; i < length; i++)
      {
         status = VIAReceiveByte(addr, &value);
         debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Read multi bytes.  addr = %xh, value = %xh.\n", addr, value);
         if (status != ERR_NO_ERROR)
         {
			debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Read multi bytes failed.  Retry.\n");
            if (retry >= MAX_RETRIES)
			{debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  All retries failed.\n");
               return status;
			}
            else
            {
               DELAY(10000);
               
            // Retry from the start because I'm not sure what might have happened
            // to the internal device address.  I think it is safer to reset the
            // internal device address with a VIAReadDataByte.   
               return I2CVIAPointerSetThenRead2(addr, pointer, retval,
                  length, retry+1);
            }
         }
		 debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  Byte read = %xh.\n", value);
         ((char FAR *)retval)[i] = value;
      }
   }

   debug_out(DETAILS_LVL,"I2CVIAPointerSetThenRead2.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
//   Writes one or more bytes of data from the I2C slave device
//   using the Byte Data Read protocol on the VIA SMBus controller
I2CSTATUS I2CVIAPointerSetThenWrite2(BYTE IN addr, BYTE IN pointer,
                                       void FAR IN *value, int IN length, 
                                       int IN retry)
{
   I2CSTATUS status;
//   int i, j;  FH3
   int i, j;
//   char buffer[256];

   
   if (length == 2)
   {
      status = VIAWriteDataWord(addr, pointer, ((short FAR *)value)[0]);
	  debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  Write WORD.  addr = %xh, pointer = %xh, value = %xh.\n", addr, pointer, ((short FAR *)value)[0]);
      if (status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  Write WORD failed.  Retry.\n");
         if (retry >= MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  All retries failed.\n");
            return status;
		 }
         else
         {
            DELAY(10000);
            return I2CVIAPointerSetThenWrite2(addr, pointer, value,
               length, retry+1);
         }
      }
   }
   else
   {
      for (i = 0; i < length; i++)
      {
         status = VIAWriteDataByte(addr, pointer, ((char FAR *)value)[i]);
         debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  Write BYTE.  addr = %xh, pointer = %xh, value = %xh.\n", addr, pointer, ((char FAR *)value)[i]);
         j = 0;
         while (status != ERR_NO_ERROR)
         {
			debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  Write BYTE failed.  Retry.\n");
            j++;
   //         sprintf(buffer, "\t\tSMBus error for Write Data Byte on %1d try.\n", j);
   //         DbgPrint(buffer);
            if (j == MAX_RETRIES)
			{
			   debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  All retries failed.\n");
               return status;
			}
            else
            {
               DELAY(10000);
               status = VIAWriteDataByte(addr, pointer, ((char FAR *)value)[i]);
            }
         }
         
      // Increment the 'command' field.  This makes multi-byte writes 
      // with length > 2 work on EEPROMs and LMxx.  
         pointer++;
      }
   }
   debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWrite2.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}


// ---------------------------------------------------------------------------
//     Read one or more bytes from the VIA using the SMB
//     Receive Data protocol
I2CSTATUS I2CVIARead2(BYTE IN addr, void FAR OUT *retval, 
                        int IN length, int IN retry)
{
   BYTE value;
   I2CSTATUS status;
   int i, j;
//   char buffer[256];

// Initialize retval
   for (i=0; i<length; i++)
      ((char FAR*)retval)[i] = 0;

   for(i = 0; i < length; i++)
   {
      status = VIAReceiveByte(addr, &value);
	  debug_out(DETAILS_LVL,"I2CVIARead2.  Read BYTE.  addr = %xh, value = %xh.\n", addr, value);
      j = 0;
      while(status != ERR_NO_ERROR)
      {
		 debug_out(DETAILS_LVL,"I2CVIARead2.  Read BYTE failed.  Retry.\n");
         ++j;
//         sprintf(buffer, "\t\tSMBus error for Receive Byte on %1d try.\n", j);
//         DbgPrint(buffer);
         if(j == MAX_RETRIES)
		 {
			debug_out(DETAILS_LVL,"I2CVIARead2.  All retries failed.\n");
            return status;
		 }
         else
         {
            DELAY(10000);
            status = VIAReceiveByte(addr, &value);
         }
      }
	  debug_out(DETAILS_LVL,"I2CVIARead2.  Byte read = %xh.\n", value);
      ((char FAR *)retval)[i] = value;
   }
   debug_out(DETAILS_LVL,"I2CVIARead2.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}



// ---------------------------------------------------------------------------
// Reads a block of data from the I2C slave device
// using the Block Data Read protocol on the ICH SMBus controller
I2CSTATUS I2CVIAPointerSetThenReadBlock2(BYTE IN addr, BYTE IN pointer,
                                         BYTE FAR OUT *retval, int OUT *length, 
                                         int IN retry)
{
   I2CSTATUS status;
//  int i;   FH3

   status = VIAReadDataBlock1(addr, pointer, retval, length);
   debug_out(DETAILS_LVL,"I2CVIAPointerSetThenReadBlock2. addr = %xh, pointer = %xh, retval = %xh, length = %dd.\n", addr, pointer, *retval, *length);
   return status;
}


// ---------------------------------------------------------------------------
I2CSTATUS I2CVIAPointerSetThenWriteBlock2(BYTE IN addr, BYTE IN pointer,
                                          BYTE FAR IN *value, int IN length, 
                                          int IN retry)
{
   I2CSTATUS status;
//  int i, j;  FH3
   status = VIAWriteDataBlock1(addr, pointer, value, length);
   debug_out(DETAILS_LVL,"I2CVIAPointerSetThenWriteBlock2. addr = %xh, pointer = %xh, retval = %xh, length = %dd.\n", addr, pointer, *value, length);
   return status;
}


// ---------------------------------------------------------------------------
I2CSTATUS I2CVIAPointerSet2(BYTE IN addr, BYTE IN command)
{
   I2CSTATUS status;
//  int i, j;  FH3  

   status = VIASendCommand(addr, command);
   debug_out(DETAILS_LVL,"I2CVIAPointerSet2. addr = %xh, command = %xh.\n", addr, command);
   return status;
}






// ---------------------------------------------------------------------------
// Implements the SMB Receive Byte protocol on the VIA
I2CSTATUS VIAReceiveByte(BYTE IN addr, BYTE FAR* OUT retval)
{
   BYTE data;
//   int i;
   I2CSTATUS status;
//   char buffer[256];


// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// then wait till bus is inactive
// ----------------------------------
   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

   debug_out(DETAILS_LVL,"VIAReceiveByte.  Configure VIASMBHSTADD for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + VIASMBHSTADD), (addr | SMBus_READ_Transaction));
   if(IsVIAFound)  DELAY(VIADELAY);
// ----------------------------------
// Setup the SMB register for 1 or more Byte Read transactions
// wait for transaction to complete
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAReceiveByte.  Configure VIASMBHSTCNT - byte.\n");
   data = 0x04;
   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), data);
   if(IsVIAFound)  DELAY(VIADELAY); 
// start read operation
   debug_out(DETAILS_LVL,"VIAReceiveByte.  Start transaction.\n");
   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), (data | VIA_START));
   
// wait for completion
   status = VIATestSMBusStatus();

// ----------------------------------
// Read value from the VIASMBHSTDAT0 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
      *retval = INP(SMBusBaseAddr + VIASMBHSTDAT0);
	   debug_out(DETAILS_LVL,"VIAReceiveByte.  VIASMBHSTDAT0 = %xh.\n", *retval);
   }


// ----------------------------------
// Clear VIASMBHSTSTA and return
// ----------------------------------
   VIACLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"VIAReceiveByte.  VIASMBHSTSTA cleared.\n");

//    sprintf(buffer, "Status after ReceiveByteStatus = %2x .\n", INP( (SMBusBaseAddr + VIASMBHSTSTA)));
    //  myPrintf(buffer);
 
   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Read protocol on the VIA SMBus controller
I2CSTATUS VIAReadDataByte(BYTE IN addr, BYTE IN pointer, BYTE FAR OUT *retval)
{
   BYTE data;
//      int i;

   I2CSTATUS status;
//   char buffer[256];

   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;
 
// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAReadDataByte.  Configure VIASMBHSTADD for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + VIASMBHSTADD), (addr | SMBus_READ_Transaction));
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAReadDataByte.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + VIASMBHSTCMD), pointer);
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
// Setup the SMB register for a Byte Data Read operation
// ----------------------------------
   data = 0x08;        //  byte transaction
//   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), data);

// Start the transaction
   debug_out(DETAILS_LVL,"VIAReadDataByte.  Configure VIASMBHSTCNT - byte data transaction and start transaction.\n");
   OUTP((SMBusBaseAddr + VIASMBHSTCNT), (data | VIA_START));
   
   status = VIATestSMBusStatus();
   
 
// ----------------------------------
// Read returned values from the VIASMBHSTDAT0/1 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
      *retval = INP(SMBusBaseAddr + VIASMBHSTDAT0);
	   debug_out(DETAILS_LVL,"VIAReadDataByte.  VIASMBHSTDAT0 = %xh.\n", *retval);
   }

   VIACLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"VIAReadDataByte.  Host registers cleared.\n");
   
//    sprintf(buffer, "Status after ReadByteStatus = %2x .\n", INP( (SMBusBaseAddr + VIASMBHSTSTA)));
    //  myPrintf(buffer);
   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Write protocol on the VIA SMBus controller
I2CSTATUS VIAWriteDataByte(BYTE IN addr, BYTE IN pointer, BYTE IN value)
{
   BYTE data;   
//   int i;

   I2CSTATUS status;
//   char buffer[256];


   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAWriteDataByte.  Configure VIASMBHSTADD for write to %xh.\n", addr);
   OUTP( (SMBusBaseAddr + VIASMBHSTADD), (addr & ~SMBus_READ_Transaction));
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAWriteDataByte.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + VIASMBHSTCMD), pointer);
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
// Setup the SMB register for a Byte Data Write operation and initiate transaction
// ----------------------------------
   data = 0x08;
//   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), data);
  
   debug_out(DETAILS_LVL,"VIAWriteDataByte.  Write %xh (data to be written) to VIASMBHSTDAT0.\n", value);
   OUTP( (SMBusBaseAddr + VIASMBHSTDAT0), value);
   if(IsVIAFound)  DELAY(VIADELAY);   

// Start the transaction
   debug_out(DETAILS_LVL,"VIAWriteDataByte.  Configure VIASMBHSTCNT - byte data transaction and start trasaction.\n");
   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), (data | VIA_START)); 
   
   status = VIATestSMBusStatus();
   

// ----------------------------------
// Clear VIASMBHSTSTA
// ----------------------------------
   VIACLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"VIAWriteDataByte.  VIASMBHSTSTA cleared.\n");
   
//    sprintf(buffer, "Status after WriteByteStatus = %2x .\n", INP( (SMBusBaseAddr + VIASMBHSTSTA)));
    //  myPrintf(buffer);
 
   return status;
}


// ---------------------------------------------------------------------------
//     Implements the SMB Send Byte protocol on the VIA
I2CSTATUS VIASendByte(BYTE IN addr, BYTE IN value)
{
   BYTE data;
//      int i;

   I2CSTATUS status;
//   char buffer[256];

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

   debug_out(DETAILS_LVL,"VIASendByte.  Configure VIASMBHSTADD for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + VIASMBHSTADD), (addr & ~SMBus_READ_Transaction));
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
// Setup the SMB register for 1 or more Byte Read transactions
// ----------------------------------
   data = 0x04;
//   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), data);
   //  DELAY(VIADELAY); 
   debug_out(DETAILS_LVL,"VIASendByte.  Write %xh (data to be written) to VIASMBHSTDAT0.\n", value);
   OUTP( (SMBusBaseAddr + VIASMBHSTDAT0), value);
   if(IsVIAFound)  DELAY(VIADELAY);   
// Start write operation
   debug_out(DETAILS_LVL,"VIASendByte.  Configure VIASMBHSTCNT - byte transaction and start transaction.\n");
   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), (data | VIA_START));

// Wait for completion
   status = VIATestSMBusStatus();
   
   VIACLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"VIASendByte. VIASMBHSTSTA cleared.\n");

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Read protocol on the VIA SMBus controller
I2CSTATUS VIAReadDataWord(BYTE IN addr, BYTE IN pointer, short FAR OUT *retval)
{
   BYTE data;
//      int i;

   I2CSTATUS status;
//   char buffer[256];

   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;
 
// ----------------------------------
// Setup the SMB register for the target device address and a read operation
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAReadDataWord.  Configure VIASMBHSTADD for read from %xh.\n", addr);
   OUTP( (SMBusBaseAddr + VIASMBHSTADD), (addr | SMBus_READ_Transaction));
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAReadDataWord.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + VIASMBHSTCMD), pointer);
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
// Setup the SMB register for a Byte Data Read operation
// not clear if start bit and command protocol bits can be set together
// so start transaction is done separately
// ----------------------------------
   data = 0x0c;        //  word transaction
//   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), data);
   //  DELAY(VIADELAY);
// Start the transaction
   debug_out(DETAILS_LVL,"VIAReadDataWord.  Configure VIASMBHSTCNT - word data transaction and start transaction.\n");
   OUTP((SMBusBaseAddr + VIASMBHSTCNT), (data | VIA_START));
   
   status = VIATestSMBusStatus();
   
//    sprintf(buffer, "Status after ReadWordStatus = %2x .\n", INP( (SMBusBaseAddr + VIASMBHSTSTA)));
    //  myPrintf(buffer);
 
// ----------------------------------
// Read returned values from the VIASMBHSTDAT0/1 registers
// ----------------------------------
   if (status == ERR_NO_ERROR)
   {
      ((char FAR*)retval)[0] = INP(SMBusBaseAddr + VIASMBHSTDAT0);
	   debug_out(DETAILS_LVL,"VIAReadDataWord.  VIASMBHSTDAT0 (lower 8bits of WORD) = %xh.\n",((char FAR*)retval)[0]);
      ((char FAR*)retval)[1] = INP(SMBusBaseAddr + VIASMBHSTDAT1);
	   debug_out(DETAILS_LVL,"VIAReadDataWord.  VIASMBHSTDAT1 (upper 8bits of WORD) = %xh.\n",((char FAR*)retval)[1]);
   }
   
   VIACLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"VIAReadDataWord.  Host registers cleared.\n");
      

   return status;
}


// ---------------------------------------------------------------------------
//   Implements the Byte Data Write protocol on the VIA SMBus controller
I2CSTATUS VIAWriteDataWord(BYTE IN addr, BYTE IN pointer, short IN value)
{
   BYTE data;
   I2CSTATUS status;
//   char buffer[256];
//      int i;

   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

// ----------------------------------
// Setup the SMB register for the target device address and a write operation
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAWriteDataWord.  Configure VIASMBHSTADD for write to %xh.\n", addr);
   OUTP( (SMBusBaseAddr + VIASMBHSTADD), (addr & ~SMBus_READ_Transaction));
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
//  Setup the SMB register with the command code
// ----------------------------------
   debug_out(DETAILS_LVL,"VIAWriteDataWord.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", pointer);
   OUTP( (SMBusBaseAddr + VIASMBHSTCMD), pointer);
   if(IsVIAFound)  DELAY(VIADELAY);

// ----------------------------------
// Setup the SMB register for a Byte Data Write operation and initiate transaction
// ----------------------------------
   data = 0x0c;
//   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), data);
//   if(IsVIAFound)  DELAY(VIADELAY);   
   OUTP( (SMBusBaseAddr + VIASMBHSTDAT0), ((char FAR *)&value)[0]);
   debug_out(DETAILS_LVL,"VIAWriteDataWord.  Write %xh (lower 8bits of WORD) into VIASMBHSTDAT0.\n", ((char FAR *)&value)[0]);
   if(IsVIAFound)  DELAY(VIADELAY);   
   OUTP( (SMBusBaseAddr + VIASMBHSTDAT1), ((char FAR *)&value)[1]);
   debug_out(DETAILS_LVL,"VIAWriteDataWord.  Write %xh (upper 8bits of WORD) into VIASMBHSTDAT1.\n", ((char FAR *)&value)[1]);
   if(IsVIAFound)  DELAY(VIADELAY);   
// Start the transaction
   debug_out(DETAILS_LVL,"VIAWriteDataWord.  Configure VIASMBHSTCNT - word data transaction and start transaction.\n");
   OUTP( (SMBusBaseAddr + VIASMBHSTCNT), (data | VIA_START));
   
   status = VIATestSMBusStatus();

// ----------------------------------
// Clear VIASMBHSTSTA
// ----------------------------------
   VIACLEAR_HOST_STATUS();
   debug_out(DETAILS_LVL,"VIAWriteDataWord.  VIASMBHSTSTA cleared.\n");
   
//    sprintf(buffer, "Status after WriteWordStatus = %2x .\n", INP( (SMBusBaseAddr + VIASMBHSTSTA)));
//    myPrintf(buffer);
 
   return status;
}

/* Reads one byte of data in block form into data[0] */
I2CSTATUS VIAReadDataBlock1 (BYTE IN address, BYTE IN command, BYTE OUT *data, int *length)
{
//   int error,i,mask,y,j,size;
//   unsigned int i;
   int i;
//   long int timeout = 650;     FH3
   I2CSTATUS status;


   status = VIABlockWaitForSMBus();
   if (status != ERR_NO_ERROR)
       return status;
   
   // Clear all status bits
   debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Clear VIASMBHSTSTS.\n");
   OUTP(SMBusBaseAddr + VIASMBHSTSTS, INP(SMBusBaseAddr + VIASMBHSTSTS));

   // Setup the slave device address
   debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Configure VIASMBHSTADD for read from %xh.\n", address);
   OUTP(SMBusBaseAddr + VIASMBHSTADD, (address | 0x01));

   // Setup the offset to read from
   debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", command);
   OUTP(SMBusBaseAddr + VIASMBHSTCMD, command);

   // Start the command
   debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Configure VIASMBHSTCNT - block transaction and start transaction.\n");
   OUTP(SMBusBaseAddr + VIASMBHSTCNT, (0x14 | VIA_START));

   BLOCKDELAY(30000);

   i = 0;
   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
   while ((status & 0x01) && (i != MAX_RETRIES))
	{
	   BLOCKDELAY(30000);
		status = INP(SMBusBaseAddr + VIASMBHSTSTS);
		i++;
	}

	if (i == MAX_RETRIES)
	{
		debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Transaction failed.  All retries unsuccessful.  Return ERR_TIMEOUT.\n");
		return ERR_TIMEOUT;
	}

	status = INP(SMBusBaseAddr + VIASMBHSTSTS);
	if (status & 0x1C)
	{
		OUTP(SMBusBaseAddr + VIASMBHSTSTS, INP(SMBusBaseAddr + VIASMBHSTSTS));
	    OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0);
	   if (status & 0x10)
	   {
		   debug_out(DETAILS_LVL,"VIAReadDataBlock1.Error occurred.  Return ERR_VIASMB_ERROR.\n");
		   return ERR_VIASMB_ERROR;
	   }
	   else if (status & 0x08)
	   {
		   debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Error occurred.  Return ERR_COLLISION.\n");
		   return ERR_COLLISION;
	   }
	   else if (status & 0x04)
	   {
		   debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Error occurred.  Return ERR_TIMEOUT_HOST\n");
		   return ERR_TIMEOUT_HOST;
	   }
	}

	//Get number of bytes read;
	*length = INP(SMBusBaseAddr + VIASMBHSTDAT0);
	debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Number of bytes read = %dd.\n", *length);

	// Reset block counter
	INP(SMBusBaseAddr + VIASMBHSTCNT);

	// Read the FIFO
	i = 0;
	while (i < *length)
	{
		//Get next data byte
		data[i] = INP(SMBusBaseAddr + VIASMBBLKDAT);
		debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Next data byte = %xh.\n", data[i]);
		i++;
	}

	// Clear status
	OUTP(SMBusBaseAddr + VIASMBHSTSTS, INP(SMBusBaseAddr + VIASMBHSTSTS));
	OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0);
    OUTP(SMBusBaseAddr + VIASMBHSTDAT0, 0);
	debug_out(DETAILS_LVL,"VIAReadDataBlock1.  Host registers cleared.\n");
//   VIACLEAR_HOST_STATUS();

	debug_out(DETAILS_LVL,"VIAReadDataBlock1.  End func - success.  Return ERR_NO_ERROR.\n");
	return ERR_NO_ERROR;
}



/* Writes one byte of data in block form from data */
I2CSTATUS VIAWriteDataBlock1 (BYTE address, BYTE command, BYTE *data, int length)
{
//   int error,i,mask,j;
//   unsigned int i;
   int i;
   I2CSTATUS status;

   // put an upper and lower boundary on count
   if (length > 32)
      length = 32;
   if (length < 0)
      length = 0;

   status = VIABlockWaitForSMBus();
   if (status != ERR_NO_ERROR)
	  return status;

   // Clear all status bits
   OUTP(SMBusBaseAddr + VIASMBHSTSTS, INP(SMBusBaseAddr + VIASMBHSTSTS));
   debug_out(DETAILS_LVL,"VIAWriteDataBlock1. VIASMBHSTSTS cleared.\n");

   // Set up the slave device address
   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Configure VIASMBHSTADD to write to %xh.\n", address);
   OUTP(SMBusBaseAddr + VIASMBHSTADD, (address & 0xFE));

   // Setup the write command
   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", command);
   OUTP(SMBusBaseAddr + VIASMBHSTCMD, command);

   // Setup the number of bytes to write as 1
   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Write %dd (bytes to be written) to VIASMBHSTDAT0.\n", length);
   OUTP(SMBusBaseAddr + VIASMBHSTDAT0, length); 

   // Reset Block Counter
   INP(SMBusBaseAddr + VIASMBHSTCNT); 

   i = 0;
   while (i < length)
   {
      OUTP(SMBusBaseAddr + VIASMBBLKDAT, data[i]);
	  debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Write %xh (data to be wriiten) to VIASMBBLKDAT.\n", data[i]);
	  BLOCKDELAY(30000);
      i++;
   }

   // Start the data block
   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Configure VIASMBHSTCNT - block transaction and start transaction.\n");
   OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0x14 | VIA_START);
   BLOCKDELAY(30000);
  
   // Wait for a response
   BLOCKDELAY(30000);

   i = 0;
   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
   while ((status & 0x01) && (i != MAX_RETRIES))
   {
       BLOCKDELAY(30000);

	   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
	   i++;
   }

   if (i == MAX_RETRIES)
   {
	   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Error occurred.  Return ERR_TIMEOUT.\n");
	   return ERR_TIMEOUT;
   }

   BLOCKDELAY(30000);

   // Check for errors.
   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
   if ((status & 0x1C) == 0x00)
   {
	   //Clear Status
	   OUTP(SMBusBaseAddr + VIASMBHSTSTS, status);
	   OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0);
	   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Host registers cleared.\n");
   }
   else
   {
	   OUTP(SMBusBaseAddr + VIASMBHSTSTS, status);
	   OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0);
	   debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  Error occurred. Host registers cleared.  Return error code.\n");

	   return status;
   }

    debug_out(DETAILS_LVL,"VIAWriteDataBlock1.  End func - success.  Return ERR_NO_ERROR.\n");
    return ERR_NO_ERROR;
}



/* Sends a command */
I2CSTATUS VIASendCommand (BYTE IN address, BYTE IN command)
{
//   int error,i,mask;
   int i;
//  long int timeout = 650;   FH3
   I2CSTATUS status;

	DELAY(30000);

   status = VIAWaitForSMBus();
   if (status != ERR_NO_ERROR)
      return status;

   // Clear all status bits
   OUTP(SMBusBaseAddr + VIASMBHSTSTS, INP(SMBusBaseAddr + VIASMBHSTSTS));
   debug_out(DETAILS_LVL,"VIASendCommand.  VIASMBHSTSTS cleared.\n");

   // Set up the slave device address
   debug_out(DETAILS_LVL,"VIASendCommand.  Configure VIASMBHSTADD for write to %xh.\n", address);
   OUTP(SMBusBaseAddr + VIASMBHSTADD, (address & 0xFE));

   // Setup the write command
   debug_out(DETAILS_LVL,"VIASendCommand.  Configure VIASMBHSTCMD with offset.  offset = %xh.\n", command);
   OUTP(SMBusBaseAddr + VIASMBHSTCMD, command);

   // Start the data block
   debug_out(DETAILS_LVL,"VIASendCommand.  Configure VIASMBHSTCNT - byte transaction and start transaction.\n");
   OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0x04 | VIA_START);
   DELAY(10000);
  
   // Wait for a response
   i = 0;
   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
   while ((status & 0x01) && (i != MAX_RETRIES))
   {
	   DELAY(10000);
	   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
	   i++;
   }

   if (i == MAX_RETRIES)
   {
	   debug_out(DETAILS_LVL,"VIASendCommand.  Error occurred.  Return ERR_TIMEOUT.\n");
	   return ERR_TIMEOUT;
   }

   DELAY(10000);

   // Check for errors.
   status = INP(SMBusBaseAddr + VIASMBHSTSTS);
   OUTP(SMBusBaseAddr + VIASMBHSTSTS, status);
   OUTP(SMBusBaseAddr + VIASMBHSTCNT, 0);
   if (!((status & 0x1C) == 0x00))
   {
	   if (status & 0x10)
	   {
		   debug_out(DETAILS_LVL,"VIASendCommand.  Error occurred.  Return ERR_VIASMB_ERROR.\n");
		   return ERR_VIASMB_ERROR;
	   }
	   else if (status & 0x08)
	   {
		   debug_out(DETAILS_LVL,"VIASendCommand.  Error occurred.  Return ERR_COLLISION.\n");
		   return ERR_COLLISION;
	   }
	   else if (status & 0x04)
	   {
		   debug_out(DETAILS_LVL,"VIASendCommand.  Error occurred.  Return ERR_TIMEOUT_HOST.\n");
		   return ERR_TIMEOUT_HOST;
	   }
   }

   debug_out(DETAILS_LVL,"VIASendCommand.  End func - success.  Return ERR_NO_ERROR.\n");
   return ERR_NO_ERROR;
}
